/*== <R&S  NGMO 2> =====================================================================*/
/* LabWindows/CVI Instrument Driver                                                     */
/* Original Release:                                                                    */
/* By: Klaus Theissen                                                                   */
/* Should you have any technical questions please contact the hotline of                */
/* Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center                        */
/*                                                                                      */
/* e-mail: CustomerSupport@rohde-schwarz.com                                            */
/* For modification history see ChangeLog                                               */
/*======================================================================================*/

/*= INCLUDES ===========================================================================*/
#include <visa.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "rsngmo.h"

/*= DEFINES ============================================================================*/
#define rsngmo_REVISION     "Rev 1.09, 08/2012, CVI 2009"    // Instrument driver revision
#define BUFFER_SIZE         1024L                           // File I/O buffer size 
    
 
/*= MACROS =============================================================================*/
#define CHECKERR(fCal) if (rsngmo_status = (fCal), rsngmo_status < VI_SUCCESS) \
                            return rsngmo_status; else

#define CHECKPARM(fCal, errCode) if (rsngmo_status = (fCal), rsngmo_status < VI_SUCCESS) {\
                            		if ((rsngmo_status > VI_ERROR_PARAMETER1) && (rsngmo_status < VI_ERROR_INSTR_PARAMETER9)) \
                            			return errCode; \
                            		else \
		                      	    	return rsngmo_status; }\
		                      	 else

/****************************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS ================================================*/
/****************************************************************************************/
static ViString channelArr[] = {"A", "B", "DVMA", "DVMB", VI_NULL};
static ViString stateArr[] = {"OFF", "ON", VI_NULL};
static ViString dataFormatArr[] = {"ASC", "LONG", "SRE", "DRE", VI_NULL};
static ViString dataOrderArr[] = {"NORM", "SWAP", VI_NULL}; 
static ViString setupArr[] = {"RST", "SAV1", "SAV2", "SAV3", "SAV4", "SAV5", "SAV6", "SAV7", "SAV8", "SAV9", "LAST", VI_NULL};
static ViString parityArr[] = {"NONE", "ODD", "EVEN", VI_NULL};
static ViString handshakeArr[] = {"NONE", "XON", "HARDWARE", VI_NULL};
static ViString delimiterArr[] = {"CR", "LF", VI_NULL};
static ViString bandwidthArr[] = {"HIGH", "LOW", VI_NULL};
static ViString limitTypeArr[] = {"LIMIT", "TRIP", VI_NULL};
static ViString functionArr[] = {"AVERAGE", "PEAK", "MIN", "HIGH", "LOW", "RMS", "VOLTAGE", "CURRENT", "DVM", VI_NULL};
static ViString currRangeArr[] = {"HIGH", "MEDIUM", "LOW", "AUTO", VI_NULL};
static ViString triggerStateArr[] = {"NONE", "TRIGGERED", "READY", "TIMEOUT", VI_NULL};
static ViString pulseChanArr[] = {"CURRENT", "DVM", VI_NULL};
static ViString triggerRangeArr[] = {"HIGH", "LOW", "DVM", VI_NULL};
static ViString triggerSourceArr[] = {"INT", "EXT", VI_NULL};
static ViString triggerSlopeArr[] = {"POS", "NEG", VI_NULL};
static ViString registerArr[] = {"*SRE", "*ESE", ":STAT:QUES:ENAB", ":STAT:MEAS:ENAB", ":STAT:OPER:ENAB", VI_NULL};
static ViString getRegisterArr[] = {"*SRE?", "*ESE?", ":STAT:QUES:ENAB?", ":STAT:MEAS:ENAB?", ":STAT:OPER:ENAB?",
								 "*STB?", "*ESR?", ":STAT:QUES:COND?", ":STAT:MEAS:COND?", ":STAT:OPER:COND?",
								 ":STAT:QUES?", ":STAT:MEAS?", ":STAT:OPER?", VI_NULL};
static ViUInt16 attrParityArr[] = {VI_ASRL_PAR_NONE, VI_ASRL_PAR_ODD, VI_ASRL_PAR_EVEN, VI_NULL};
static ViUInt16 attrHandshakeArr[] = {VI_ASRL_FLOW_NONE, VI_ASRL_FLOW_XON_XOFF, VI_ASRL_FLOW_RTS_CTS, VI_NULL}; 
static ViReal64 minTriggerRangeArr[] = {1.0E-4, 1.0E-5, -5.999, VI_NULL};
static ViReal64 maxTriggerRangeArr[] = {7.0, 5.0E-1, 25.0, VI_NULL}; 
static ViString triggerArr[] = {"*ATRG", "*BTRG", "*TRG", VI_NULL};
static ViString armTriggerArr[] = {"*AARM", "*BARM", "*ARM", VI_NULL};

/****************************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  =======================================*/
/****************************************************************************************/
/* rsngmo_stringValPair is used in the rsngmo_errorMessage function						*/
/* rsngmo_statusDataRanges is used to track session dependent status & ranges			*/
/*======================================================================================*/
typedef struct  rsngmo_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rsngmo_tStringValPair;
 
    /*=CHANGE:=============================================================*/
    /* Change to reflect the global status variables that your driver needs*/
    /* to keep track of.  For example trigger mode of each session         */
    /*=====================================================================*/
struct rsngmo_statusDataRanges {
    ViChar		instrumentFWRevision[BUFFER_SIZE];  
    ViInt32		instrumentFWRevisionNum;
    ViBoolean	checkStatus;
};

typedef struct rsngmo_statusDataRanges *rsngmo_instrRange;

/****************************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) ============================*/
/****************************************************************************************/
ViBoolean rsngmo_invalidViBooleanRange (ViBoolean val);
ViBoolean rsngmo_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rsngmo_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);

ViStatus rsngmo_defaultInstrSetup (ViSession instrumentHandle);  
ViStatus rsngmo_initCleanUp (ViSession openRMSession, ViPSession openinstrumentHandle, ViStatus currentStatus);
ViStatus _reset (ViSession instrumentHandle);  
ViStatus rsngmo_statusQuery (ViSession instrumentHandle);

ViStatus rsngmo_setViBoolean (ViSession instrumentHandle, ViString cmd, ViBoolean value);    
ViStatus rsngmo_getViBoolean (ViSession instrumentHandle, ViString cmd, ViBoolean *value);
ViStatus rsngmo_setEnum (ViSession instrumentHandle, ViString cmd, ViInt32 value, ViString _VI_FAR list[]);
ViStatus rsngmo_getEnum (ViSession instrumentHandle, ViString cmd, ViInt32 *value, ViString _VI_FAR list[]);						  
ViStatus rsngmo_setViInt32 (ViSession instrumentHandle, ViString cmd, 
						     ViInt32 value, ViInt32 min, ViInt32 max);
ViStatus rsngmo_getViInt32 (ViSession instrumentHandle, ViString cmd, ViInt32 *value);   
ViStatus rsngmo_setViReal64 (ViSession instrumentHandle, ViString cmd, ViReal64 value, 
							  ViReal64 min, ViReal64 max);
ViStatus rsngmo_getViReal64 (ViSession instrumentHandle, ViString cmd, ViReal64 *value);
ViStatus rsngmo_setNode (ViSession instrumentHandle, ViString node);
ViStatus rsngmo_getNode (ViSession instrumentHandle, ViString command, ViChar response[]);
ViStatus rsngmo_setViBooleanChan (ViSession instrumentHandle, ViString cmd,
									ViInt32 channel, ViBoolean value);
ViStatus rsngmo_getViBooleanChan (ViSession instrumentHandle, ViString cmd, 
								ViInt32 channel, ViBoolean *value);
ViStatus rsngmo_setEnumChan (ViSession instrumentHandle, ViString cmd, ViInt32 value, 
						 ViInt32 channel, ViString _VI_FAR list[]);
ViStatus rsngmo_getEnumChan (ViSession instrumentHandle, ViString cmd,
						 ViInt32 channel, ViInt32 *value, ViString _VI_FAR list[]);
ViStatus rsngmo_setViInt32Chan (ViSession instrumentHandle, ViString cmd, 
							ViInt32 channel, ViInt32 value, ViInt32 min, ViInt32 max);
ViStatus rsngmo_getViInt32Chan (ViSession instrumentHandle, ViString cmd, 
							ViInt32 channel, ViInt32 *value);  
ViStatus rsngmo_setViReal64Chan (ViSession instrumentHandle, ViString cmd, ViInt32 channel,
							ViReal64 value, ViReal64 min, ViReal64 max);
ViStatus rsngmo_getViReal64Chan (ViSession instrumentHandle, ViString cmd, ViInt32 channel, 
							ViReal64 *value);
ViStatus rsngmo_setNodeChan (ViSession instrumentHandle, ViString cmd, ViInt32 channel);
ViStatus rsngmo_getNodeChan (ViSession instrumentHandle, ViString cmd, ViInt32 channel,
							 ViChar response[]);
ViStatus rsngmo_checkIndexInList (ViSession instrumentHandle, ViInt32 index, ViString list[], 
								   ViStatus statusIfError);
ViStatus rsngmo_getIndexFromString (ViSession vi, ViString string, 
									 ViString *list, ViInt32 *index);
ViStatus rsngmo_setChannelPart (ViString cmd, ViInt32 channel, ViChar retCmd[]);
void rsngmo_delay (ViReal64 delay);
ViStatus _setActiveDisplayChannel (ViSession instrumentHandle, ViInt32 channel);
ViStatus _setDisplayState (ViSession instrumentHandle, ViBoolean state); 
ViStatus _setBeeperState (ViSession instrumentHandle, ViInt32 channel, ViBoolean state);
ViStatus _setDataFormat (ViSession instrumentHandle, ViInt32 format);
ViStatus _getDataFormat (ViSession instrumentHandle, ViInt32 *format);  
ViStatus _setByteOrder (ViSession instrumentHandle, ViInt32 order); 
ViStatus _getByteOrder (ViSession instrumentHandle, ViInt32 *order);
ViStatus _setSerialState (ViSession instrumentHandle,  ViBoolean state);
ViStatus _setBaudRate (ViSession instrumentHandle, ViInt32 baud);
ViStatus _setDataLength (ViSession instrumentHandle, ViInt32 datalength);  
ViStatus _setParity (ViSession instrumentHandle, ViInt32 parity);
ViStatus _setStopBits (ViSession instrumentHandle, ViInt32 stopbits);
ViStatus _setHandshake (ViSession instrumentHandle, ViInt32 handshake);
ViStatus _setDelimiter (ViSession instrumentHandle, ViInt32 delimiter);
ViStatus _setOutputState (ViSession instrumentHandle, ViInt32 channel, ViBoolean state);
ViStatus _setOutputBandwidth (ViSession instrumentHandle, ViInt32 channel, ViInt32 bandwidth);
ViStatus _setOutputImpedance (ViSession instrumentHandle, ViInt32 channel, ViReal64 impedance);
ViStatus _setVoltageLevel (ViSession instrumentHandle, ViInt32 channel, ViReal64 voltageLevel);
ViStatus _setVoltageProtection (ViSession instrumentHandle, ViInt32 channel,
								ViReal64 protectionLevel);
ViStatus _setVoltageMaxSetting (ViSession instrumentHandle, ViInt32 channel, ViReal64 max);
ViStatus _setCurrentLimit (ViSession instrumentHandle, ViInt32 channel, ViReal64 limit);
ViStatus _setCurrentLimitMaxSetting (ViSession instrumentHandle, ViInt32 channel, ViReal64 max);
ViStatus _setCurrentLimitType (ViSession instrumentHandle, ViInt32 channel, ViInt32 type);
ViStatus _setMeasurementFunction (ViSession instrumentHandle, ViInt32 channel, ViInt32 function);
ViStatus _setCurrentMeasurementRange (ViSession instrumentHandle, ViInt32 channel, ViInt32 range);
ViStatus _setMeasurementInterval (ViSession instrumentHandle, ViInt32 channel, ViReal64 interval);
ViStatus _setAverageCount (ViSession instrumentHandle, ViInt32 channel, ViInt32 averageCount);
ViStatus _setPulseChannel (ViSession instrumentHandle, ViInt32 channel, ViInt32 pulseChannel);
ViStatus _setPulseState (ViSession instrumentHandle, ViInt32 channel, ViBoolean state);
ViStatus _setPulseType (ViSession instrumentHandle, ViInt32 channel, ViInt32 type);
ViStatus _setPulseTriggerLevel (ViSession instrumentHandle, ViInt32 channel,
								ViInt32 triggerRange, ViReal64 triggerLevel);
ViStatus _setPulseTriggerSource (ViSession instrumentHandle, ViInt32 channel, ViInt32 source);
ViStatus _setPulseTriggerSlope (ViSession instrumentHandle, ViInt32 channel, ViInt32 slope);
ViStatus _setPulseTriggerOffset (ViSession instrumentHandle, ViInt32 channel, ViInt32 offset);
ViStatus _setPulseTriggerCount (ViSession instrumentHandle, ViInt32 channel, ViInt32 triggerCount);  
ViStatus _setPulseTriggerTimeout (ViSession instrumentHandle, ViInt32 channel,ViReal64 timeout);
ViStatus _setPulseNumberOfSamples (ViSession instrumentHandle, ViInt32 channel, ViInt32 numberOfSamples);
ViStatus _setPulseSampleInterval (ViSession instrumentHandle, ViInt32 channel, ViReal64 sampleInterval);
ViStatus rsngmo_getDataArray (ViSession instrumentHandle, ViChar cmd[], ViInt32 channel,
							ViInt32 function, ViInt32 numberPoints, ViReal64 *array,
							ViInt32 *retCount);
ViStatus rsngmo_getDataSingle (ViSession instrumentHandle, ViChar cmd[], ViInt32 channel,
							ViInt32 function, ViReal64 *value);
ViStatus _setStatusRegister (ViSession instrumentHandle, ViInt32 reg, ViInt32 value); 

/****************************************************************************************/
/*============ INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ==================*/
/****************************************************************************************/

/****************************************************************************************/
/*============== USER-CALLABLE FUNCTIONS (Exportable Functions) ========================*/
/****************************************************************************************/

/*======================================================================================*/
/* Function: Initialize																	*/
/* Purpose:  This function opens the instrument, initializes the instrument				*/
/*           to a known status and resets the instrument.								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_init (ViRsrc resourceName, ViBoolean IDQuery, 
								ViBoolean resetDevice, ViSession *instrumentHandle)

{
    ViStatus rsngmo_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViChar rdBuffer[BUFFER_SIZE];
    ViUInt16 interface = VI_INTF_GPIB;

    /*- Check input parameter ranges ----------------------------------------*/
    if (rsngmo_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rsngmo_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;
    
    /*- Open instrument session ---------------------------------------------*/
    if ((rsngmo_status = viOpenDefaultRM (&rmSession)) < 0)
        return rsngmo_status;
   
    if ((rsngmo_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG, VI_NULL, instrumentHandle)) < 0) {
        viClose (rmSession);
        return rsngmo_status;
    }

	/*- Discard all warnings; when it is used GPIB interface viOpen returns
	    warning 0x3FFF0077 VI_WARN_CONFIG_NLOADED */
	    
	rsngmo_status = VI_SUCCESS;
    
    /*- Configure VISA Formatted I/O ----------------------------------------*/

    if ((rsngmo_status = viSetAttribute (*instrumentHandle, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
  
    if ((rsngmo_status = viSetBuf (*instrumentHandle, VI_READ_BUF|VI_WRITE_BUF, 25000)) < 0)
            return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
  
    if ((rsngmo_status = viSetAttribute (*instrumentHandle, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
    
    if ((rsngmo_status = viSetAttribute (*instrumentHandle, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
 
    /* Serial interface support */
    if ((rsngmo_status = viGetAttribute (*instrumentHandle, VI_ATTR_INTF_TYPE, &interface)) < 0)
        return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
 	
 	/* Serial Port */
    if (interface == VI_INTF_ASRL){
	    if ((rsngmo_status = viPrintf (*instrumentHandle, ":SYST:COMM:SER:DEL LF\r\n")) < 0)
	        return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status); 
	    if ((rsngmo_status = viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_TERMCHAR)) < 0)
	    	return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status); 
	    if ((rsngmo_status = viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_END_IN,VI_ASRL_END_TERMCHAR)) < 0)
	    	return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status); 
	    if ((rsngmo_status = viSetAttribute (*instrumentHandle, VI_ATTR_TERMCHAR_EN, VI_TRUE)) < 0)
	    	return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status); 
	 // Line Feed is fixed
	    if ((rsngmo_status = viSetAttribute (*instrumentHandle, VI_ATTR_TERMCHAR,10)) < 0)
	    	return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
        if ((rsngmo_status = viWrite (*instrumentHandle, (ViPBuf)"*CLS", 4, VI_NULL)) < 0)
            return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
	    
 	} 
 	
    /*- Identification Query ------------------------------------------------*/
    if (IDQuery){
        if ((rsngmo_status = viWrite (*instrumentHandle, (ViPBuf)"*IDN?", 5, VI_NULL)) < 0)
            return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
        
        if ((rsngmo_status = viRead (*instrumentHandle, (ViPBuf)rdBuffer, 100, VI_NULL)) < 0)
            return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
        
        if (strncmp (rdBuffer, "ROHDE&SCHWARZ,NGMO", 18))
            return rsngmo_initCleanUp (rmSession, instrumentHandle, VI_ERROR_FAIL_ID_QUERY);
    }  
    
    /*- Reset --------------------------------------------------------------*/
    if (resetDevice){
    	if ((rsngmo_status = rsngmo_reset (*instrumentHandle)) < 0)
    		return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
    }		
    else
    	if ((rsngmo_defaultInstrSetup (*instrumentHandle)) < 0)
    		return rsngmo_initCleanUp (rmSession, instrumentHandle, rsngmo_status);
	
	CHECKERR (rsngmo_statusQuery (*instrumentHandle));
		
    return rsngmo_status;
}



/*======================================================================================*/
/* Function: Configure Front Panel														*/
/* Purpose:  This function sets the status of the front panel and displayed channel		*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_configureFrontPanel (ViSession instrumentHandle,
                                              ViBoolean state, ViInt32 displayedChannel)
{
    ViStatus rsngmo_status = VI_SUCCESS;
    
    CHECKERR (_setDisplayState (instrumentHandle, state)); 
    CHECKPARM (_setActiveDisplayChannel (instrumentHandle, displayedChannel), VI_ERROR_PARAMETER3);
    
    return rsngmo_status;
    
}

/*======================================================================================*/
/* Function: Set/Get Active Display Channel												*/
/* Purpose:  These functions sets/gets the active channel displayed on display			*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setActiveDisplayChannel (ViSession instrumentHandle, ViInt32 channel)
{
	return _setActiveDisplayChannel (instrumentHandle, channel);
}

ViStatus _VI_FUNC rsngmo_getActiveDisplayChannel (ViSession instrumentHandle, ViInt32 *channel)
{
	return rsngmo_getEnum (instrumentHandle, ":DISP:CHAN?", channel, channelArr);  
}

/*======================================================================================*/
/* Function: Set/Get Display State														*/
/* Purpose:  These functions set/get the state of display								*/
/*======================================================================================*/

ViStatus _VI_FUNC rsngmo_setDisplayState (ViSession instrumentHandle, ViBoolean state)
{
	return _setDisplayState (instrumentHandle, state); 
}

ViStatus _VI_FUNC rsngmo_getDisplayState (ViSession instrumentHandle, ViBoolean *state)
{
	return rsngmo_getViBoolean (instrumentHandle, ":DISP:ENAB?", state); 
}

/*======================================================================================*/
/* Function: Set/Get Output State Key Control											*/
/* Purpose:  These functions set/get the key for the power output for BOTH Channels		*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setCommonOutputState (ViSession instrumentHandle, ViBoolean state)
{
	return rsngmo_setViBoolean (instrumentHandle, ":CONF:COMM:OUTP:ONOF", state);
}

ViStatus _VI_FUNC rsngmo_getCommonOutputState (ViSession instrumentHandle, ViBoolean *state)
{
	return rsngmo_getViBoolean (instrumentHandle, ":CONF:COMM:OUTP:ONOF?", state);
}

/*======================================================================================*/
/* Function: Set/Get Beeper State														*/
/* Purpose:  This function sets/gets the beeper state									*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setBeeperState (ViSession instrumentHandle, ViInt32 channel, ViBoolean state)
{
	return _setBeeperState (instrumentHandle, channel, state); 
}

ViStatus _VI_FUNC rsngmo_getBeeperState (ViSession instrumentHandle, ViInt32 channel, ViBoolean *state)
{
	return rsngmo_getViBooleanChan (instrumentHandle, ":CONF:%s:CURR:LIM:BEEP?", channel, state);
}


/*======================================================================================*/
/* Function: Set/Get Power On Setup														*/
/* Purpose:  This function sets/gets the default setting for power up					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPowerOnSetup (ViSession instrumentHandle, ViInt32 setupNumber)
{
	return rsngmo_setEnum (instrumentHandle, ":SYST:POS", setupNumber, setupArr);	    
}

ViStatus _VI_FUNC rsngmo_getPowerOnSetup (ViSession instrumentHandle, ViInt32 *setupNumber)
{
	return rsngmo_getEnum (instrumentHandle, ":SYST:POS?", setupNumber, setupArr);	    
}

/*======================================================================================*/
/* Function: Configure Serial Setup														*/
/* Purpose:  This function configures the serial interface								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_configureSerialSetup (ViSession instrumentHandle,
                                               ViBoolean serialPort,
                                               ViInt32 datalength, ViInt32 baud,
                                               ViInt32 parity, ViInt32 stopbits,
                                               ViInt32 handshake)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	
	CHECKPARM (_setDataLength (instrumentHandle, datalength), VI_ERROR_PARAMETER3);
	CHECKPARM (_setBaudRate (instrumentHandle, baud), VI_ERROR_PARAMETER4);
	CHECKPARM (_setParity (instrumentHandle, parity), VI_ERROR_PARAMETER5);
	CHECKPARM (_setStopBits (instrumentHandle, stopbits), VI_ERROR_PARAMETER6); 	
	CHECKPARM (_setHandshake (instrumentHandle, handshake), VI_ERROR_PARAMETER7); 
	CHECKERR (_setDelimiter (instrumentHandle,  RSNGMO_SER_DEL_LF));
	CHECKERR (_setSerialState (instrumentHandle, serialPort)); 
	
	return rsngmo_status;
}

/*======================================================================================*/
/* Function: Set/Get Serial State														*/
/* Purpose:  This function sets/gets the serial interface state							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setSerialState (ViSession instrumentHandle, ViBoolean state)
{
	return _setSerialState (instrumentHandle, state);  
}

ViStatus _VI_FUNC rsngmo_getSerialState (ViSession instrumentHandle, ViBoolean *state)
{
	return rsngmo_getViBoolean (instrumentHandle, ":SYST:COMM:SER:ENAB?", state);  
}

/*======================================================================================*/
/* Function: Set/Get Baud Rate															*/
/* Purpose:  This function sets/gets the serial interface baud rate						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setBaudRate (ViSession instrumentHandle, ViInt32 baudRate)
{
	return _setBaudRate (instrumentHandle, baudRate);  
}

ViStatus _VI_FUNC rsngmo_getBaudRate (ViSession instrumentHandle, ViInt32 *baudRate)
{
	return rsngmo_getViInt32 (instrumentHandle, ":SYST:COMM:SER:BAUD?", baudRate);  
}

/*======================================================================================*/
/* Function: Set/Get Data Length														*/
/* Purpose:  This function sets/gets the serial interface data length					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setDatalength (ViSession instrumentHandle, ViInt32 dataLength)
{
	return _setDataLength (instrumentHandle, dataLength); 
}

ViStatus _VI_FUNC rsngmo_getDatalength (ViSession instrumentHandle, ViInt32 *dataLength)
{
	return rsngmo_getViInt32(instrumentHandle, ":SYST:COMM:SER:DAT?", dataLength);  
}

/*======================================================================================*/
/* Function: Set/Get Parity																*/
/* Purpose:  This function sets/gets the serial interface parity						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setParity (ViSession instrumentHandle, ViInt32 parity)
{
    return _setParity (instrumentHandle, parity);
}

ViStatus _VI_FUNC rsngmo_getParity (ViSession instrumentHandle, ViInt32 *parity)
{
	return rsngmo_getEnum (instrumentHandle, ":SYST:COMM:SER:PAR?", parity, parityArr);  
}

/*======================================================================================*/
/* Function: Set/Get Stop Bits															*/
/* Purpose:  This function sets/gets the stop bits										*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setStopBits (ViSession instrumentHandle, ViInt32 stopbits)
{
	return _setStopBits (instrumentHandle, stopbits); 
}

ViStatus _VI_FUNC rsngmo_getStopBits (ViSession instrumentHandle, ViInt32 *stopbits)
{
	return rsngmo_getViInt32 (instrumentHandle, ":SYST:COMM:SER:STOP?", stopbits); 	    
}

/*======================================================================================*/
/* Function: Set/Get Handshake															*/
/* Purpose:  This function sets/gets the handshake										*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setHandshake (ViSession instrumentHandle, ViInt32 handshake)
{
	return _setHandshake (instrumentHandle, handshake);    
}

ViStatus _VI_FUNC rsngmo_getHandshake (ViSession instrumentHandle, ViInt32 *handshake)
{
	return rsngmo_getEnum (instrumentHandle, ":SYST:COMM:SER:HAND?", handshake, handshakeArr); 
}

/*======================================================================================*/
/* Function: Set/Get IEEE Address														*/
/* Purpose:  This function sets/gets the IEEE Address									*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setGPIBAddress (ViSession instrumentHandle, ViInt32 address)
{
	return rsngmo_setViInt32 (instrumentHandle, ":SYST:COMM:IEEE:ADDR", address, 1, 30); 
}

ViStatus _VI_FUNC rsngmo_getGPIBAddress (ViSession instrumentHandle, ViInt32 *address)
{
	return rsngmo_getViInt32 (instrumentHandle, ":SYST:COMM:IEEE:ADDR?", address); 
}

/*======================================================================================*/
/* Function: Configure Output															*/
/* Purpose:  This function configures selected output									*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_configureOutput (ViSession instrumentHandle,
                                          ViInt32 channel, ViBoolean state,
                                          ViInt32 bandwidth, ViReal64 impedance)
{
   ViStatus rsngmo_status = VI_SUCCESS;
   
   CHECKERR (_setOutputState (instrumentHandle, channel, state));  
   CHECKPARM (_setOutputBandwidth (instrumentHandle, channel, bandwidth), VI_ERROR_PARAMETER4);
   CHECKPARM (_setOutputImpedance (instrumentHandle, channel, impedance), VI_ERROR_PARAMETER5);
   
   return rsngmo_status;
}

/*======================================================================================*/
/* Function: Set/Get Output State														*/
/* Purpose:  This function sets/gets the state of selected output						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setOutputState (ViSession instrumentHandle, 
										ViInt32 channel, ViBoolean state)
{
	return _setOutputState (instrumentHandle, channel, state);    
}

ViStatus _VI_FUNC rsngmo_getOutputState (ViSession instrumentHandle,
                                         ViInt32 channel, ViBoolean *state)
{
	return rsngmo_getViBooleanChan (instrumentHandle, ":OUTP:%s:STAT?", channel, state); 
}

/*======================================================================================*/
/* Function: Set/Get Output Bandwidth													*/
/* Purpose:  This function sets/gets the output bandwidth								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setOutputBandwidth (ViSession instrumentHandle,
                                             ViInt32 channel, ViInt32 bandwidth)
{
	return _setOutputBandwidth (instrumentHandle, channel, bandwidth);  
}

ViStatus _VI_FUNC rsngmo_getOutputBandwidth (ViSession instrumentHandle,
                                             ViInt32 channel, ViInt32 *bandwidth)
{
	return rsngmo_getEnumChan (instrumentHandle, ":OUTP:%s:BAND?", channel, bandwidth, bandwidthArr);    
}

/*======================================================================================*/
/* Function: Set/Get Output Impedance													*/
/* Purpose:  This function sets/gets the output impedance								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setOutputImpedance (ViSession instrumentHandle,
                                             ViInt32 channel, ViReal64 impedance)
{
	return _setOutputImpedance (instrumentHandle, channel, impedance);    
}

ViStatus _VI_FUNC rsngmo_getOutputImpedance (ViSession instrumentHandle, 
											 ViInt32 channel, ViReal64 *impedance)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":OUTP:%s:IMP?", channel, impedance);    
}

/*======================================================================================*/
/* Function: Set/Get Output Realy State													*/
/* Purpose:  This function sets/gets the output relay state								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setOutputRelayState (ViSession instrumentHandle,
                                              ViInt32 relay, ViBoolean state)
{
	ViChar buffer[BUFFER_SIZE];
	
	if (rsngmo_invalidViInt32Range (relay, 1, 4))
		return VI_ERROR_PARAMETER2;
	
	if (rsngmo_invalidViBooleanRange (state))
		return VI_ERROR_PARAMETER3;
	
	sprintf (buffer, "OUTP:REL%d", relay); 
	return rsngmo_setViBoolean (instrumentHandle, buffer, state);    
}

ViStatus _VI_FUNC rsngmo_getOutputRelayState (ViSession instrumentHandle,
                                              ViInt32 relay, ViBoolean *state)
{
	ViChar buffer[BUFFER_SIZE];
	
	if (rsngmo_invalidViInt32Range (relay, 1, 4))
		return VI_ERROR_PARAMETER2;
	
	if (state == VI_NULL)
		return VI_ERROR_PARAMETER3;
	
	sprintf (buffer, "OUTP:REL%d?", relay); 
	return rsngmo_getViBoolean (instrumentHandle, buffer, state);    
}

/*======================================================================================*/
/* Function: Configure Source															*/
/* Purpose:  This function configures the source voltage level, current limit and		*/
/*			 current limit type.														*/	
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_configureSource (ViSession instrumentHandle,
                                          ViInt32 channel, ViReal64 voltageLevel,
                                          ViReal64 currentLimit,
                                          ViInt32 currentLimitType)
{
	ViStatus rsngmo_status = VI_NULL;
	
	CHECKERR (_setVoltageLevel (instrumentHandle, channel, voltageLevel));
	CHECKPARM (_setCurrentLimit (instrumentHandle, channel, currentLimit), VI_ERROR_PARAMETER4);
	CHECKPARM (_setCurrentLimitType (instrumentHandle, channel, currentLimitType),
										VI_ERROR_PARAMETER5);
	
	return rsngmo_status;
}

/*======================================================================================*/
/* Function: Configure Source Protection												*/
/* Purpose:  This function configures the the voltage protection value  for voltage		*/
/*			 and the max. settings and current limit.									*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_configureSourceProtection (ViSession instrumentHandle,
                                                   ViInt32 channel,
                                                   ViReal64 voltageProtection,
                                                   ViReal64 maximumVoltage,
                                                   ViReal64 maximumCurrentLimit)
{
	ViStatus rsngmo_status = VI_SUCCESS;      
	
	CHECKERR (_setVoltageProtection (instrumentHandle, channel, voltageProtection));
	CHECKPARM (_setVoltageMaxSetting (instrumentHandle, channel, maximumVoltage), 
				VI_ERROR_PARAMETER4);
	CHECKPARM (_setCurrentLimitMaxSetting (instrumentHandle, channel, maximumCurrentLimit), 
				VI_ERROR_PARAMETER5);

	return rsngmo_status;			
}

/*======================================================================================*/
/* Function: Set/Get Voltage Level														*/
/* Purpose:  This function sets/gets the source voltage amplitude						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setVoltage (ViSession instrumentHandle, ViInt32 channel,
											ViReal64 voltageLevel)
{
	return _setVoltageLevel (instrumentHandle, channel, voltageLevel);  
}

ViStatus _VI_FUNC rsngmo_getVoltage (ViSession instrumentHandle, ViInt32 channel,
                                          ViReal64 *voltageLevel)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":SOUR:%s:VOLT?", channel, voltageLevel);     
}

/*======================================================================================*/
/* Function: Set/Get Voltage Protection													*/
/* Purpose:  This function sets/gets the source voltage protection						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setVoltageProtection (ViSession instrumentHandle, ViInt32 channel,
                                               ViReal64 protectionLevel)
{
	return _setVoltageProtection (instrumentHandle, channel, protectionLevel);    
}

ViStatus _VI_FUNC rsngmo_getVoltageProtection (ViSession instrumentHandle, ViInt32 channel,
                                               ViReal64 *protectionLevel)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":SOUR:%s:VOLT:PROT?", channel, protectionLevel);    
}

/*======================================================================================*/
/* Function: Get Voltage Protection State												*/
/* Purpose:  This function gets the source voltage protection state						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_getVoltageProtectionState (ViSession instrumentHandle, 
													ViInt32 channel, ViInt32 *state)
{
	return rsngmo_getViInt32Chan (instrumentHandle, ":SOUR:%s:VOLT:PROT:STATE?", channel,
									state);
}

/*======================================================================================*/
/* Function: Set/Get Voltage Max  Setting												*/
/* Purpose:  This function sets/gets the max voltage settable value						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setVoltageMaxSetting (ViSession instrumentHandle,
                                               ViInt32 channel, ViReal64 max)
{
	return _setVoltageMaxSetting (instrumentHandle, channel, max);    
}

ViStatus _VI_FUNC rsngmo_getVoltageMaxSetting (ViSession instrumentHandle, 
                                               ViInt32 channel, ViReal64 *max)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":SOUR:%s:VOLT:MAXS?", channel, max); 	
}

/*======================================================================================*/
/* Function: Set/Get Current Limit														*/
/* Purpose:  This function sets/gets the current limit									*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setCurrentLimit (ViSession instrumentHandle, ViInt32 channel, 
											ViReal64 limit)
{
	return _setCurrentLimit (instrumentHandle, channel, limit);    
}

ViStatus _VI_FUNC rsngmo_getCurrentLimit (ViSession instrumentHandle, ViInt32 channel,
                                          ViReal64 *limit)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":SOUR:%s:CURR?", channel, limit);    
}

/*======================================================================================*/
/* Function: Set/Get Current Limit	Max Setting											*/
/* Purpose:  This function sets/gets the maximum settable current limit					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setCurrentLimitMaxSetting (ViSession instrumentHandle,
                                                    ViInt32 channel, ViReal64 max)
{
	return _setCurrentLimitMaxSetting (instrumentHandle, channel, max);    
}

ViStatus _VI_FUNC rsngmo_getCurrentLimitMaxSetting (ViSession instrumentHandle,
                                                    ViInt32 channel, ViReal64 *max)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":SOUR:%s:CURR:MAXS?", channel, max);     
}

/*======================================================================================*/
/* Function: Set/Get Current Limit Type													*/
/* Purpose:  This function sets/gets the current limit type								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setCurrentLimitType (ViSession instrumentHandle, ViInt32 channel,
                                               ViInt32 type)
{
	return _setCurrentLimitType (instrumentHandle, channel, type);    
}

ViStatus _VI_FUNC rsngmo_getCurrentLimitType (ViSession instrumentHandle,
                                              ViInt32 channel, ViInt32 *type)
{
	return rsngmo_getEnumChan (instrumentHandle, ":SOUR:%s:CURR:TYPE?", channel, 
								type, limitTypeArr);    
}

/*======================================================================================*/
/* Function: Get Current Limit State													*/
/* Purpose:  This function gets the current limit state									*/
/*======================================================================================*/

ViStatus _VI_FUNC rsngmo_getCurrentLimitState (ViSession instrumentHandle, ViInt32 channel,
                                                ViInt32 *state)
{
	return rsngmo_getViInt32Chan (instrumentHandle, ":SOUR:%s:CURR:STAT?", channel, state);
}

/*======================================================================================*/
/* Function: Arm Trigger            													*/
/* Purpose:  This function arms trigger of specified channel   							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_armTrigger (ViSession instrumentHandle, ViInt32 channel)
{
	ViStatus rsngmo_status = VI_SUCCESS;    
	rsngmo_instrRange instrPtr;
	
	CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
	
	if (instrPtr->instrumentFWRevisionNum < 1003)
		return RSNGMO_ERROR_FUNCTION_UNAVAILABLE;
	else
		return viPrintf (instrumentHandle, "%s", armTriggerArr[channel]);
}

/*======================================================================================*/
/* Function: Trigger                													*/
/* Purpose:  This function triggers specified channel   								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_trigger (ViSession instrumentHandle, ViInt32 channel)
{
	return viPrintf (instrumentHandle, "%s", triggerArr[channel]);
}

/*======================================================================================*/
/* Function: Configure Measurement														*/
/* Purpose:  This function configures the measurement									*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_configureMeasurement (ViSession instrumentHandle,
                                               ViInt32 channel, ViInt32 function,
                                               ViInt32 currentRange,
                                               ViInt32 averageCount,
                                               ViReal64 interval)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	
	CHECKERR (_setMeasurementFunction (instrumentHandle, channel, function));
	CHECKPARM (_setCurrentMeasurementRange (instrumentHandle, channel, currentRange),
				VI_ERROR_PARAMETER4);
	CHECKPARM (_setAverageCount (instrumentHandle, channel, averageCount), 
				VI_ERROR_PARAMETER5);	
	CHECKPARM (_setMeasurementInterval (instrumentHandle, channel, interval), 
				VI_ERROR_PARAMETER6);	
	
	return rsngmo_status;					
}

/*======================================================================================*/
/* Function: Configure Pulse Measurement												*/
/* Purpose:  This function configures the pulse measurement								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_configurePulseMeasurement (ViSession instrumentHandle,
                                                    ViInt32 channel,
                                                    ViInt32 pulseChannel,
                                                    ViInt32 pulseType,
                                                    ViInt32 numberOfSamples,
                                                    ViReal64 sampleInterval,
                                                    ViBoolean state)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	
	CHECKERR (_setPulseChannel (instrumentHandle, channel, pulseChannel));
	CHECKPARM (_setPulseType (instrumentHandle, channel, pulseType),
				VI_ERROR_PARAMETER4);
	CHECKPARM (_setPulseNumberOfSamples (instrumentHandle, channel, numberOfSamples), 
				VI_ERROR_PARAMETER5);	
	CHECKPARM (_setPulseSampleInterval (instrumentHandle, channel, sampleInterval), 
				VI_ERROR_PARAMETER6);	
	CHECKPARM (_setPulseState (instrumentHandle, channel, state), 
				VI_ERROR_PARAMETER7);	
	
	return rsngmo_status;					
}

/*======================================================================================*/
/* Function: Configure Pulse Trigger													*/
/* Purpose:  This function configures the pulse trigger									*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_configurePulseTrigger (ViSession instrumentHandle,
                                                ViInt32 channel, ViInt32 source,
                                                ViInt32 triggerRange, ViReal64 triggerLevel,
                                                ViInt32 triggerCount,
                                                ViInt32 slope, ViInt32 offset,
                                                ViReal64 timeout)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	
	CHECKERR (_setPulseTriggerSource (instrumentHandle, channel, source));
	
	if (rsngmo_invalidViInt32Range (triggerRange, 0, 2))
		return VI_ERROR_PARAMETER4;
	CHECKPARM (_setPulseTriggerLevel (instrumentHandle, channel, triggerRange, triggerLevel),
				VI_ERROR_PARAMETER5);
	CHECKPARM (_setPulseTriggerCount (instrumentHandle, channel, triggerCount), 
				VI_ERROR_PARAMETER6);	
	CHECKPARM (_setPulseTriggerSlope (instrumentHandle, channel, slope), 
				VI_ERROR_PARAMETER7);	
	CHECKPARM (_setPulseTriggerOffset (instrumentHandle, channel, offset), 
				VI_ERROR_PARAMETER8);	
	CHECKPARM (_setPulseTriggerTimeout (instrumentHandle, channel, timeout), 
				VI_ERROR_INSTR_PARAMETER9);	
	
	return rsngmo_status;					
}

/*======================================================================================*/
/* Function: Set/Get Measurement Function												*/
/* Purpose:  This function sets/gets the measurement function							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setMeasurementFunction (ViSession instrumentHandle, ViInt32 channel, 
                                                 ViInt32 function)
{
	return _setMeasurementFunction (instrumentHandle, channel, function);    
}

ViStatus _VI_FUNC rsngmo_getMeasurementFunction (ViSession instrumentHandle, ViInt32 channel, 
                                                 ViInt32 *function)
{
	return rsngmo_getEnumChan (instrumentHandle, ":SENS:%s:FUNC?", channel, function, functionArr);
}

/*======================================================================================*/
/* Function: Set/Get Current Measurement Range											*/
/* Purpose:  This function sets/gets expected current measurement range					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setCurrentMeasurementRange (ViSession instrumentHandle,
                                                     ViInt32 channel, ViInt32 range)
{
	return _setCurrentMeasurementRange (instrumentHandle, channel, range);
}

ViStatus _VI_FUNC rsngmo_getCurrentMeasurementRange (ViSession instrumentHandle,
                                                     ViInt32 channel, ViInt32 *range)
{
	return rsngmo_getEnumChan (instrumentHandle, ":SENS:%s:CURR:RANGE?", channel, range,
							currRangeArr);
}

/*======================================================================================*/
/* Function: Set/Get Measurement Interval												*/
/* Purpose:  This function sets/gets measurement interval for voltage and current		*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setMeasurementInterval (ViSession instrumentHandle,
                                                 ViInt32 channel, ViReal64 interval)
{
	return _setMeasurementInterval (instrumentHandle, channel, interval);   
}

ViStatus _VI_FUNC rsngmo_getMeasurementInterval (ViSession instrumentHandle,
                                                 ViInt32 channel, ViReal64 *interval)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":SENS:%s:MEAS:INT?", channel, interval);     
}

/*======================================================================================*/
/* Function: Set/Get Average Count														*/
/* Purpose:  This function sets/gets the measure average count							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setAverageCount (ViSession instrumentHandle,
                                          ViInt32 channel, ViInt32 averageCount)
{
	return _setAverageCount (instrumentHandle, channel, averageCount);    
}

ViStatus _VI_FUNC rsngmo_getAverageCount (ViSession instrumentHandle,
										  ViInt32 channel, ViInt32 *averageCount)
{
	return rsngmo_getViInt32Chan (instrumentHandle, ":SENS:%s:AVER:COUN?", channel,
								averageCount);     
}

/*======================================================================================*/
/* Function: Get Pulse Trigger State													*/
/* Purpose:  This function returns the current trigger status.							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_getPulseTriggerState (ViSession instrumentHandle,
                                               ViInt32 channel, ViInt32 *state)
{
	return rsngmo_getEnumChan (instrumentHandle, ":SENS:%s:PULS:TRIG:STAT?", channel, 
								state, triggerStateArr);
}

/*======================================================================================*/
/* Function: Set/Get Pulse Channel														*/
/* Purpose:  This function sets/gets the pulse measurement channel.						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseChannel (ViSession instrumentHandle,
                                          ViInt32 channel, ViInt32 pulseChannel)
{
	return _setPulseChannel (instrumentHandle, channel, pulseChannel);      
}

ViStatus _VI_FUNC rsngmo_getPulseChannel (ViSession instrumentHandle,
                                          ViInt32 channel, ViInt32 *pulseChannel)
{
	return rsngmo_getEnumChan (instrumentHandle, ":SENS:%s:PULS:CHAN?", channel,
									pulseChannel, pulseChanArr);    
}

/*======================================================================================*/
/* Function: Set/Get Pulse State														*/
/* Purpose:  This function sets/gets the pulse measurement state.						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseState (ViSession instrumentHandle,
                                        ViInt32 channel, ViBoolean state)
{
	return _setPulseState (instrumentHandle, channel, state);    
}

ViStatus _VI_FUNC rsngmo_getPulseState (ViSession instrumentHandle,
                                        ViInt32 channel, ViBoolean *state)
{
	return rsngmo_getViBooleanChan (instrumentHandle, ":SENS:%s:PULS:STAR?", channel, state);    
}

/*======================================================================================*/
/* Function: Set/Get Pulse Type															*/
/* Purpose:  This function sets/gets the pulse measurement type.						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseMeasurementType (ViSession instrumentHandle,
                                       ViInt32 channel, ViInt32 type)
{
	return _setPulseType (instrumentHandle, channel, type);    
}

ViStatus _VI_FUNC rsngmo_getPulseMeasurementType (ViSession instrumentHandle, 
										ViInt32 channel, ViInt32 *type)
{
	return rsngmo_getEnumChan (instrumentHandle, ":SENS:%s:PULS:TYPE?", channel, type, functionArr);    
}

/*======================================================================================*/
/* Function: Set/Get Pulse Trigger Level												*/
/* Purpose:  This function sets/gets the pulse trigger level							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseTriggerLevel (ViSession instrumentHandle, ViInt32 channel,
                                               ViInt32 triggerRange, ViReal64 triggerLevel)
{
	return _setPulseTriggerLevel (instrumentHandle, channel, triggerRange, triggerLevel);    
}

ViStatus _VI_FUNC rsngmo_getPulseTriggerLevel (ViSession instrumentHandle, ViInt32 channel,
                                               ViInt32 triggerRange, ViReal64 *triggerLevel)
{
	
	ViStatus rsngmo_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
	
	if (rsngmo_invalidViInt32Range (triggerRange, 0, 2))
		return VI_ERROR_PARAMETER3;
	CHECKERR (rsngmo_setChannelPart (":SENS:%s:PULS:TRIG:LEV:", channel, buffer)); 
	strcat (buffer, triggerRangeArr[triggerRange]);
	strcat (buffer, "?");
	CHECKPARM (rsngmo_getViReal64 (instrumentHandle, buffer, triggerLevel), VI_ERROR_PARAMETER4);
	
	return rsngmo_status;			  
}

/*======================================================================================*/
/* Function: Set/Get Pulse Trigger Source												*/
/* Purpose:  This function sets/gets the pulse trigger source							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseTriggerSource (ViSession instrumentHandle,
                                                ViInt32 channel, ViInt32 source)
{
	return _setPulseTriggerSource (instrumentHandle, channel, source);    
}

ViStatus _VI_FUNC rsngmo_getPulseTriggerSource (ViSession instrumentHandle,
                                                ViInt32 channel, ViInt32 *source)
{
	return rsngmo_getEnumChan (instrumentHandle, ":SENS:%s:PULS:TRIG:SOUR?", channel,
								source, triggerSourceArr);   
}

/*======================================================================================*/
/* Function: Set/Get Pulse Trigger Slope												*/
/* Purpose:  This function sets/gets the pulse trigger slope							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseTriggerSlope (ViSession instrumentHandle,
                                               ViInt32 channel, ViInt32 slope)
{
	return _setPulseTriggerSlope (instrumentHandle, channel, slope);    
}

ViStatus _VI_FUNC rsngmo_getPulseTriggerSlope (ViSession instrumentHandle,
                                               ViInt32 channel, ViInt32 *slope)
{
	return rsngmo_getEnumChan (instrumentHandle, ":SENS:%s:PULS:TRIG:SLOP?", channel,    
								slope, triggerSlopeArr);
}

/*======================================================================================*/
/* Function: Set/Get Pulse Trigger Offset												*/
/* Purpose:  This function sets/gets the pulse trigger offset							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseTriggerOffset (ViSession instrumentHandle,
                                                ViInt32 channel, ViInt32 offset)
{
	return _setPulseTriggerOffset (instrumentHandle, channel, offset);    
}

ViStatus _VI_FUNC rsngmo_getPulseTriggerOffset (ViSession instrumentHandle,
                                                ViInt32 channel, ViInt32 *offset)
{
	return rsngmo_getViInt32Chan (instrumentHandle, ":SENS:%s:PULS:TRIG:OFFS?", channel,
									offset);
}

/*======================================================================================*/
/* Function: Set/Get Pulse Trigger Count												*/
/* Purpose:  This function sets/gets the pulse trigger count							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseTriggerCount (ViSession instrumentHandle,
                                               ViInt32 channel, ViInt32 triggerCount)
{
	return _setPulseTriggerCount (instrumentHandle, channel, triggerCount);       
}

ViStatus _VI_FUNC rsngmo_getPulseTriggerCount (ViSession instrumentHandle,
                                               ViInt32 channel, ViInt32 *count)
{
	return rsngmo_getViInt32Chan (instrumentHandle, ":SENS:%s:PULS:TRIG:COUN?", channel,
									count);
}

/*======================================================================================*/
/* Function: Set/Get Pulse Trigger Timeout												*/
/* Purpose:  This function sets/gets the pulse trigger timrout							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseTriggerTimeout (ViSession instrumentHandle,
                                                 ViInt32 channel,ViReal64 timeout)
{
	return _setPulseTriggerTimeout (instrumentHandle, channel, timeout);     
}

ViStatus _VI_FUNC rsngmo_getPulseTriggerTimeout (ViSession instrumentHandle,
                                                 ViInt32 channel, ViReal64 *timeout)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":SENS:%s:PULS:TRIG:TIM?", channel,
									timeout);
}

/*======================================================================================*/
/* Function: Set/Get Pulse Number Of Samples											*/
/* Purpose:  This function sets/gets the pulse number of samples						*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseNumberOfSamples (ViSession instrumentHandle,
                                                  ViInt32 channel, ViInt32 numberOfSamples)
{
	return _setPulseNumberOfSamples (instrumentHandle, channel, numberOfSamples);      
}

ViStatus _VI_FUNC rsngmo_getPulseNumberOfSamples (ViSession instrumentHandle,
                                                  ViInt32 channel, ViInt32 *numberOfSamples)
{
	return rsngmo_getViInt32Chan (instrumentHandle, ":SENS:%s:PULS:SAMP:LENG?", channel, 
								  numberOfSamples);	
}

/*======================================================================================*/
/* Function: Set/Get Pulse Sample Interval												*/
/* Purpose:  This function sets/gets the pulse sample interval							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setPulseSampleInterval (ViSession instrumentHandle,
                                                 ViInt32 channel, ViReal64 sampleInterval)
{
	return _setPulseSampleInterval (instrumentHandle, channel, sampleInterval);    
}

ViStatus _VI_FUNC rsngmo_getPulseSampleInterval (ViSession instrumentHandle,
                                                 ViInt32 channel, ViReal64 *sampleInterval)
{
	return rsngmo_getViReal64Chan (instrumentHandle, ":SENS:%s:PULS:SAMP:INT?", channel, 
									sampleInterval);	
}

/*======================================================================================*/
/* Function: Fetch Measurement															*/
/* Purpose:  This function fetchs the value							 					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_fetchMeasurement (ViSession instrumentHandle, ViInt32 channel, ViReal64 *value)
{
	if (value == VI_NULL)
		return VI_ERROR_PARAMETER3;
		
	return (rsngmo_getDataSingle (instrumentHandle, "FETC:%s", channel, VI_NULL, value));
}

/*======================================================================================*/
/* Function: Fetch Measurement Array													*/
/* Purpose:  This function fetchs the array of readings				 					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_fetchMeasurementArray (ViSession instrumentHandle,
                                                ViInt32 channel,
                                                ViInt32 arraySize,
                                                ViReal64 data[],
                                                ViInt32 *actualNumberOfSamples)
{
	if (rsngmo_invalidViInt32Range (arraySize, 0, 5000))
	    return VI_ERROR_PARAMETER3;
	    
	if (data == VI_NULL)
		return VI_ERROR_PARAMETER4;
	    
	return rsngmo_getDataArray (instrumentHandle, "FETC:%s", channel, VI_NULL, arraySize,
								data, actualNumberOfSamples);
}

/*======================================================================================*/
/* Function: Read Measurement															*/
/* Purpose:  This function reads the value							 					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_readMeasurement (ViSession instrumentHandle,
                                          ViInt32 channel, ViReal64 *value)
{
	if (value == VI_NULL)
		return VI_ERROR_PARAMETER3;
	
	return rsngmo_getDataSingle (instrumentHandle, "READ:%s", channel, VI_NULL, value);     
}

/*======================================================================================*/
/* Function: Read Measurement Array														*/
/* Purpose:  This function reads the array of readings				 					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_readMeasurementArray (ViSession instrumentHandle,
                                               ViInt32 channel,
                                               ViInt32 arraySize, ViReal64 data[],
                                               ViInt32 *actualNumberOfSamples)
{
	if (rsngmo_invalidViInt32Range (arraySize, 0, 5000))
	    return VI_ERROR_PARAMETER3;
	
	if (data == VI_NULL)
		return VI_ERROR_PARAMETER4;
	
	return rsngmo_getDataArray (instrumentHandle, "READ:%s", channel, VI_NULL, arraySize,
								data, actualNumberOfSamples);
}

/*======================================================================================*/
/* Function: Measure																	*/
/* Purpose:  This function measures function						 					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_measure (ViSession instrumentHandle, ViInt32 channel,
                                  ViInt32 function, ViReal64 *value)
{
	return rsngmo_getDataSingle (instrumentHandle, "MEAS:%s", channel, function, value);
}

/*======================================================================================*/
/* Function: Sets/Gets Status Register													*/
/* Purpose:  This function writes/reads a register of the NGMO2							*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_setStatusRegister (ViSession instrumentHandle,
                                            ViInt32 reg, ViInt32 value)
{
	return _setStatusRegister (instrumentHandle, reg, value);
}

ViStatus _VI_FUNC rsngmo_getStatusRegister (ViSession instrumentHandle,
                                            ViInt32 reg, ViInt32 *value)
{
	if (rsngmo_invalidViInt32Range (reg, 0, 12))
		return VI_ERROR_PARAMETER2;
	
	return rsngmo_getViInt32 (instrumentHandle, getRegisterArr[reg], value);	    
}

/*======================================================================================*/
/* Function: Preset Status Registers													*/
/* Purpose:  This function returns status registers to default state					*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_presetStatusRegisters (ViSession instrumentHandle)
{
	return rsngmo_setNode (instrumentHandle, ":STAT:PRES");    
}

/*======================================================================================*/
/* Function: Clear All Registers                       	                              	*/
/* Purpose:  This function clear all instrument registers								*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_clearAllRegisters (ViSession instrumentHandle)
{
	ViStatus rsngmo_status = VI_SUCCESS;  
	ViInt32 i;

	for (i=RSNGMO_REG_SRE; i<RSNGMO_REG_OPER_ENAB; i++)
		CHECKERR (_setStatusRegister (instrumentHandle, i, 0));
		
	return rsngmo_status;	
}

/*======================================================================================*/
/* Function: Write To Instr		 		                                            	*/
/* Purpose:  This function writes a command string to the instrument.        			*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_writeToInstr (ViSession instrumentHandle, ViChar writeBuffer[])
{
	ViStatus rsngmo_status = VI_SUCCESS;
	
	CHECKERR (viPrintf (instrumentHandle, "%s", writeBuffer));     
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Read Instr Data 		                                            		*/
/* Purpose:  This function writes a command string to the instrument.        			*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_readInstrData (ViSession instrumentHandle, ViInt32 numberBytesToRead,
										ViChar readBuffer[], ViInt32 *numBytesRead)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	
	*numBytesRead = 0L;     
	CHECKERR (viRead (instrumentHandle, (ViPBuf)readBuffer, numberBytesToRead, (ViPUInt32)numBytesRead));
	
	return rsngmo_statusQuery (instrumentHandle);
    
}

/*======================================================================================*/
/* Function: Reset																		*/
/* Purpose:  This function resets the instrument and clears all event					*/
/*			 registers and error queues.												*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_reset (ViSession instrumentHandle)
{
    return _reset (instrumentHandle);
}

/*======================================================================================*/
/* Function: Self Test																	*/
/* Purpose:  This function executes the instrument self-test and returns				*/
/*           the result. 																*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_self_test (ViSession instrumentHandle,
                                   ViInt32 *selfTestResult)
{
	ViStatus rsngmo_status = VI_SUCCESS;

	CHECKERR (viPrintf (instrumentHandle, "*TST?"));
	CHECKERR (viScanf (instrumentHandle, "%ld", selfTestResult));
	return rsngmo_statusQuery (instrumentHandle); 
}

/*======================================================================================*/
/* Function: Error Query                                                     			*/
/* Purpose:  This function queries the instrument error queue, and returns   			*/
/*           the result. 													 			*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_error_query (ViSession instrumentHandle,
                                     ViInt32 *errorCode, _VI_FAR ViChar errorMessage[])
{
	ViStatus rsngmo_status = VI_SUCCESS;
	
	if (errorCode == VI_NULL) 
		return VI_ERROR_PARAMETER2;
	if (errorMessage == VI_NULL)
		return VI_ERROR_PARAMETER3;
		
	CHECKERR (viPrintf (instrumentHandle, ":SYST:ERR?"));
	/*=NOTE:=============================================================*/
    /* Parse the error message and return the Error Code and Error String. */
    /* This code makes the following assumption: The String returned from  */
    /* the instrument is in the format set forth in the SCPI 1994 Command  */
    /* Reference which is <Error/Event Number,"Error/Event Descriptor">,   */
    /* for example  0,"No Error" or -410,"Query Interrupted".              */
    /*=====================================================================*/
	CHECKERR (viScanf (instrumentHandle, "%ld,\"%[^\"]", errorCode, errorMessage));
	
	return rsngmo_statusQuery (instrumentHandle);  
}

/*======================================================================================*/
/* Function: Clear Error Queue                                                 			*/
/* Purpose:  This function removes all errors from error queue.				   			*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_ClearErrorQueue (ViSession instrumentHandle)
{

    ViStatus rsngmo_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrumentHandle, ":SYST:CLE"));
    return rsngmo_statusQuery (instrumentHandle);     
}

/*======================================================================================*/
/* Function: Error Message                                                   			*/
/* Purpose:  This function translates the error return value from the        			*/
/*           instrument driver into a user-readable string.                  			*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_error_message (ViSession instrumentHandle, ViStatus statusCode, 
										ViChar message[])
{
    ViStatus rsngmo_status = VI_SUCCESS;
    ViInt16 i;
    static rsngmo_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {RSNGMO_ERROR_INSTRUMENT_ERROR, "ERROR: Instrument reported an error, use rsngmo_errorQuery"},
        {RSNGMO_ERROR_MEASUREMENT_TIMEOUT, "ERROR: Instrument didn't trigger to the timeout defined by parameter Timeout"},
        {RSNGMO_ERROR_TRIGGER_TIMEOUT, "ERROR: Instrument reported trigger timeout error"},
        {RSNGMO_ERROR_FUNCTION_UNAVAILABLE, "ERROR: This function is not implemented in this firmware revision"},
        {VI_NULL, VI_NULL}
    };

    rsngmo_status = viStatusDesc (instrumentHandle, statusCode, message);
    if (rsngmo_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == statusCode) {
                strcpy (message, statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        sprintf (message, "Unknown Error 0x%08lX", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    rsngmo_status = VI_SUCCESS;
    return rsngmo_status;
	    
}

/*======================================================================================*/
/* Function: Revision Query                                                  			*/
/* Purpose:  This function returns the driver and instrument revisions.      			*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_revision_query (ViSession instrumentHandle, ViChar _VI_FAR instrumentDriverRevision[],
                                         ViChar _VI_FAR firmwareRevision[])
{
	ViStatus rsngmo_status = VI_SUCCESS;    
	rsngmo_instrRange instrPtr;
	
	if (instrumentDriverRevision == VI_NULL)
		return VI_ERROR_PARAMETER2;
	if (firmwareRevision == VI_NULL)
		return VI_ERROR_PARAMETER3;
		
	CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
	strcpy (instrumentDriverRevision, rsngmo_REVISION);
	strcpy (firmwareRevision, instrPtr->instrumentFWRevision);
	
	return rsngmo_status;
}

/*======================================================================================*/
/* Function: SaveSetup                                 	                              	*/
/* Purpose:  This function saves present setup as a user defined setup				  	*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_saveSetup (ViSession instrumentHandle, ViInt32 userSetup)
{
	return rsngmo_setViInt32 (instrumentHandle, "*SAV", userSetup, 1, 9);    
}

/*======================================================================================*/
/* Function: Recall Setup                              	                              	*/
/* Purpose:  This function recalls setup of selected number. 0 - factory, 1-9 user 	  	*/
/*			 setting.																	*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_recallSetup (ViSession instrumentHandle, ViInt32 setupNumber)
{
	return rsngmo_setViInt32 (instrumentHandle, "*RCL", setupNumber, 0, 9);	    
}

/*===========================================================================*/
/* Function: Check Status                                                    */
/* Purpose:  This function switches ON/OFF state checking of the instrument  */
/*           (reading of the Standard Event Register and checking it for     */
/*           error). Nearly all driver function are using state checking.    */
/*           Switch this function to ON when debug your application. For     */
/*           better bus throughput and instruments performance switch it OFF.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsngmo_setCheckStatus (ViSession instrumentHandle, ViBoolean state)
{
    ViStatus rsngmo_status = VI_SUCCESS;
    rsngmo_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
    instrPtr -> checkStatus = state;

    return rsngmo_status;
}

ViStatus _VI_FUNC rsngmo_getCheckStatus (ViSession instrumentHandle, ViBoolean *state)
{
    ViStatus rsngmo_status = VI_SUCCESS;
    rsngmo_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
    *state = instrPtr -> checkStatus;

    return rsngmo_status;
}

/*======================================================================================*/
/* Function: Close																		*/
/* Purpose:  This function closes the instrument.										*/
/*======================================================================================*/
ViStatus _VI_FUNC rsngmo_close (ViSession instrumentHandle)
{
    rsngmo_instrRange instrPtr;
    ViSession rmSession;
    ViStatus rsngmo_status = VI_SUCCESS;

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr != NULL) 
        free (instrPtr);
    
    rsngmo_status = viClose (instrumentHandle);
    viClose (rmSession);

    return rsngmo_status;
}

/****************************************************************************************/
/*=========== UTILITY ROUTINES (Non-Exportable Functions) ==============================*/
/****************************************************************************************/

/*======================================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean										*/
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE			*/
/*           or VI_FALSE. If the value is out of range, the return value is				*/
/*           VI_TRUE, otherwise the return value is VI_FALSE.							*/
/*======================================================================================*/
ViBoolean rsngmo_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*======================================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32							*/
/* Purpose:  This function checks a long signed integer value to see if it				*/  
/*           lies between a minimum and maximum value.  If the value is out				*/
/*           of range, the return value is VI_TRUE, otherwise the return				*/
/*           value is VI_FALSE.															*/
/*======================================================================================*/
ViBoolean rsngmo_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*======================================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64								*/
/* Purpose:  This function checks a real (double) value to see if it lies				*/  
/*           between a minimum and maximum value.  If the value is out of				*/
/*           range, the return value is VI_TRUE, otherwise the return value				*/
/*           is VI_FALSE.																*/
/*======================================================================================*/
ViBoolean rsngmo_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}


/****************************************************************************************/
/*----------------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE ------------------*/
/****************************************************************************************/

/*======================================================================================*/
/* Function: Default Instrument Setup													*/
/* Purpose:  This function sends a default setup to the instrument.  This				*/
/*           function is called by the rsngmo_reset operation and by the				*/
/*           rsngmo_init function if the reset option has not been						*/
/*           selected. This function is useful for configuring any						*/
/*           instrument settings that are required by the rest of the					*/
/*           instrument driver functions such as turning headers ON or OFF				*/
/*           or using the long or short form for commands, queries, and data.			*/
/*======================================================================================*/
ViStatus rsngmo_defaultInstrSetup (ViSession instrumentHandle)
{
    ViStatus rsngmo_status = VI_SUCCESS;
    ViChar buffer[BUFFER_SIZE];
    ViChar revision[BUFFER_SIZE];
    rsngmo_instrRange instrPtr;

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
											
    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr == VI_NULL)
    {
        instrPtr = malloc (sizeof (struct rsngmo_statusDataRanges));
	    CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_USER_DATA, (ViAttrState)instrPtr));
    }

	// Enable status checking by default
    instrPtr -> checkStatus = VI_TRUE;
    
    // Determine the firmware revision
	sprintf (buffer, "*IDN?");
    CHECKERR (viWrite (instrumentHandle, (ViPBuf)buffer, (ViUInt32) strlen(buffer), VI_NULL)); 
    CHECKERR (viRead (instrumentHandle, (ViPBuf)buffer, BUFFER_SIZE, VI_NULL));
	sscanf (buffer, "%*[^,],%*[^,],%*[^,],%[^\r\n]", revision);
	instrPtr -> instrumentFWRevisionNum = 1000*(revision[0]-'0') + 100*(revision[2]-'0') + revision[3]-'0';
	strcpy (instrPtr -> instrumentFWRevision, revision);	
	sprintf (buffer, "*CLS; :STAT:PRES;");
	CHECKERR (viWrite (instrumentHandle, (ViPBuf)buffer, (ViUInt32) strlen(buffer), VI_NULL));
	
	//Set data format and byte order
	_setDataFormat (instrumentHandle, RSNGMO_FORMAT_SREAL);
	_setByteOrder (instrumentHandle, RSNGMO_BYTE_ORDER_SWAP);
	
    return rsngmo_status;
}

/*======================================================================================*/
/* Function: Initialize Clean Up														*/
/* Purpose:  This function is used only by the rsngmo_init function.  When				*/
/*           an error is detected this function is called to close the					*/
/*           open resource manager and instrument object sessions and to				*/
/*           set the instrumentHandle that is returned from rsngmo_init to				*/
/*           VI_NULL.																	*/
/*======================================================================================*/
ViStatus rsngmo_initCleanUp (ViSession openRMSession,
                    ViPSession openinstrumentHandle, ViStatus currentStatus)
{
    viClose (*openinstrumentHandle);
    viClose (openRMSession);
    *openinstrumentHandle = VI_NULL;
    
    return currentStatus;
}

/*======================================================================================*/
/* Function: Reset																		*/
/* Purpose:  This function resets the instrument and clears all event					*/
/*			 registers and error queues.												*/
/*======================================================================================*/
ViStatus _reset (ViSession instrumentHandle)
{
    ViStatus rsngmo_status = VI_SUCCESS;

    /*  Initialize the instrument to a known status.  */
    CHECKERR (viPrintf (instrumentHandle, "*RST"));
    CHECKERR (rsngmo_defaultInstrSetup (instrumentHandle));
    return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Status Query																*/
/* Purpose:  This function queries the status byte register and returns					*/
/*           the status of bit 3 (ESB).													*/
/*======================================================================================*/
ViStatus rsngmo_statusQuery (ViSession instrumentHandle)
{
    ViStatus rsngmo_status = VI_SUCCESS;
    ViInt32 esrByte;
	rsngmo_instrRange instrPtr;							 
	
	CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr)); 
    if (instrPtr -> checkStatus == VI_TRUE){
	    CHECKERR (viPrintf (instrumentHandle, "*ESR?"));
	    CHECKERR (viScanf (instrumentHandle, "%ld", &esrByte));

	    /*--------------- Check if any error or message bit was asserted -*/
	    if ((esrByte & 0x3C) != 0)
	            rsngmo_status = RSNGMO_ERROR_INSTRUMENT_ERROR;
	    else
	        rsngmo_status = VI_SUCCESS;
	}
    return rsngmo_status;
}

/*======================================================================================*/
/* Function: Set Boolean																*/
/* Purpose:  This function sets command with boolean parameter							*/
/*           parameter.																	*/
/*======================================================================================*/
ViStatus rsngmo_setViBoolean (ViSession instrumentHandle, ViString cmd, ViBoolean value)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	
	if (rsngmo_invalidViBooleanRange(value))
		return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrumentHandle, "%s %s", cmd, stateArr[value]));
	
	return rsngmo_statusQuery (instrumentHandle); 
}

/*======================================================================================*/
/* Function: Get Boolean																*/
/* Purpose:  This function gets command with boolean parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_getViBoolean (ViSession instrumentHandle, ViString cmd, ViBoolean *value)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	ViInt32		index;
	
	if (value == VI_NULL) 
		return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrumentHandle, cmd));
	CHECKERR (viScanf (instrumentHandle, "%s", buffer));
	CHECKERR (rsngmo_getIndexFromString (instrumentHandle, buffer, stateArr, &index)); 
	*value = (index == 0) ? VI_FALSE : VI_TRUE;
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Set Enum																	*/
/* Purpose:  This function sets command with enumerated parameter.						*/
/*======================================================================================*/
ViStatus rsngmo_setEnum (ViSession instrumentHandle, ViString cmd, ViInt32 value, 
						  ViString _VI_FAR list[])
{
	ViStatus	rsngmo_status = VI_SUCCESS;

	CHECKERR (rsngmo_checkIndexInList (instrumentHandle, value, list, VI_ERROR_PARAMETER2));	
	CHECKERR (viPrintf (instrumentHandle, "%s %s", cmd, list[value]));
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Get Enum																	*/
/* Purpose:  This function gets command with enumerated parameter.						*/
/*======================================================================================*/
ViStatus rsngmo_getEnum (ViSession instrumentHandle, ViString cmd, ViInt32 *value, ViString _VI_FAR list[])
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar		buffer[256];
	
	if (value == VI_NULL)
		return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrumentHandle, cmd));
	CHECKERR (viScanf (instrumentHandle, "%[^\r\n]", buffer)); 

	CHECKERR (rsngmo_getIndexFromString (instrumentHandle, buffer, list, value));
	
	return rsngmo_statusQuery (instrumentHandle);  
}

/*======================================================================================*/
/* Function: Set ViInt32																*/
/* Purpose:  This function sets command with integer parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_setViInt32 (ViSession instrumentHandle, ViString cmd, 
						     ViInt32 value, ViInt32 min, ViInt32 max)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	
	if (rsngmo_invalidViInt32Range (value, min, max))
		return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrumentHandle, "%s %ld", cmd, value));
	
	return rsngmo_statusQuery (instrumentHandle);   
}

/*======================================================================================*/
/* Function: Get ViInt32																*/
/* Purpose:  This function gets command with integer parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_getViInt32 (ViSession instrumentHandle, ViString cmd, ViInt32 *value)   
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	
	if (value == VI_NULL)
		return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrumentHandle, cmd));
	CHECKERR (viScanf (instrumentHandle, "%ld", value));
//	CHECKERR (viQueryf (instrumentHandle, cmd, "%ld", value));

	return rsngmo_statusQuery (instrumentHandle);   
}

/*======================================================================================*/
/* Function: Set ViReal64																*/
/* Purpose:  This function sets command with double parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_setViReal64 (ViSession instrumentHandle, ViString cmd, ViReal64 value, 
							  ViReal64 min, ViReal64 max)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	
	if (max - min > 1.0e-14)
		if (rsngmo_invalidViReal64Range (value, min, max))
			return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrumentHandle, "%s %Lf", cmd, value));
	
	return rsngmo_statusQuery (instrumentHandle); 
}

/*======================================================================================*/
/* Function: Get ViReal64																*/
/* Purpose:  This function gets command with double parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_getViReal64 (ViSession instrumentHandle, ViString cmd, ViReal64 *value)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	
	if (value == VI_NULL)
		return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrumentHandle, cmd));
	CHECKERR (viScanf (instrumentHandle, "%Le", value));

	return rsngmo_statusQuery (instrumentHandle); 
}

/*===========================================================================*/
/* Function: Set Node 		                                                 */
/* Purpose:  This function sets simple command node that has no parameters.  */
/*===========================================================================*/
ViStatus rsngmo_setNode (ViSession instrumentHandle, ViString node)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	
	CHECKERR (viPrintf (instrumentHandle, node));
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*===========================================================================*/
/* Function: Get Node 		                                                 */
/* Purpose:  This function gets a string response to simple query that has   */
/*           no parameters.													 */
/*===========================================================================*/
ViStatus rsngmo_getNode (ViSession instrumentHandle, ViString command, ViChar response[])
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	
	if (response == VI_NULL)
		return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrumentHandle, command));
	CHECKERR (viScanf (instrumentHandle, "%[^\r\n]", response));  

	return rsngmo_statusQuery (instrumentHandle);
}


/*======================================================================================*/
/* Function: Set Boolean with channel number											*/
/* Purpose:  This function sets command with boolean parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_setViBooleanChan (ViSession instrumentHandle, ViString cmd,
									ViInt32 channel, ViBoolean value)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
	
	if (rsngmo_invalidViBooleanRange(value))
		return VI_ERROR_PARAMETER3;

	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer));
	CHECKERR (viPrintf (instrumentHandle, "%s %s", buffer, stateArr[value]));
	
	return rsngmo_statusQuery (instrumentHandle); 
}

/*======================================================================================*/
/* Function: Get Boolean with channel number											*/
/* Purpose:  This function gets command with boolean parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_getViBooleanChan (ViSession instrumentHandle, ViString cmd, 
								ViInt32 channel, ViBoolean *value)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	ViInt32		index;
	
	if (value == VI_NULL) 
		return VI_ERROR_PARAMETER3;
	
	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer)); 
	CHECKERR (viPrintf (instrumentHandle, buffer));
	CHECKERR (viScanf (instrumentHandle, "%s", buffer));
	CHECKERR (rsngmo_getIndexFromString (instrumentHandle, buffer, stateArr, &index)); 
	*value = (index == 0) ? VI_FALSE : VI_TRUE;

	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Set Enum with channel number												*/
/* Purpose:  This function sets command with enumerated parameter.						*/
/*======================================================================================*/
ViStatus rsngmo_setEnumChan (ViSession instrumentHandle, ViString cmd, ViInt32 channel, 
							ViInt32 value, ViString _VI_FAR list[])
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	CHECKERR (rsngmo_checkIndexInList (instrumentHandle, value, list, VI_ERROR_PARAMETER3));	
	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer)); 	
	CHECKERR (viPrintf (instrumentHandle, "%s %s", buffer, list[value]));
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Get Enum with channel number												*/
/* Purpose:  This function gets command with enumerated parameter.						*/
/*======================================================================================*/
ViStatus rsngmo_getEnumChan (ViSession instrumentHandle, ViString cmd,
						 ViInt32 channel, ViInt32 *value, ViString _VI_FAR list[])
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	if (value == VI_NULL)
		return VI_ERROR_PARAMETER3;
	
	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer)); 
	CHECKERR (viPrintf (instrumentHandle, buffer));
	CHECKERR (viScanf (instrumentHandle, "%[^\r\n]", buffer)); 

	
	CHECKERR (rsngmo_getIndexFromString (instrumentHandle, buffer, list, value));
	
	return rsngmo_statusQuery (instrumentHandle);  
}

/*======================================================================================*/
/* Function: Set ViInt32 with channel number											*/
/* Purpose:  This function sets command with integer parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_setViInt32Chan (ViSession instrumentHandle, ViString cmd, 
							ViInt32 channel, ViInt32 value, ViInt32 min, ViInt32 max)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	if (rsngmo_invalidViInt32Range (value, min, max))
		return VI_ERROR_PARAMETER3;
	
	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer));
	CHECKERR (viPrintf (instrumentHandle, "%s %ld", buffer, value));
	
	return rsngmo_statusQuery (instrumentHandle);   
}

/*======================================================================================*/
/* Function: Get ViInt32 with channel number											*/
/* Purpose:  This function gets command with integer parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_getViInt32Chan (ViSession instrumentHandle, ViString cmd, 
							ViInt32 channel, ViInt32 *value)   
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	if (value == VI_NULL)
		return VI_ERROR_PARAMETER3;

	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer));
	CHECKERR (viPrintf (instrumentHandle, buffer));
	CHECKERR (viScanf (instrumentHandle, "%ld", value));


	return rsngmo_statusQuery (instrumentHandle);   
}

/*======================================================================================*/
/* Function: Set ViReal64 with channel number											*/
/* Purpose:  This function sets command with double parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_setViReal64Chan (ViSession instrumentHandle, ViString cmd, ViInt32 channel,
							ViReal64 value, ViReal64 min, ViReal64 max)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar 		buffer[BUFFER_SIZE];
	
	if (max - min > 1.0e-14)
		if (rsngmo_invalidViReal64Range (value, min, max))
			return VI_ERROR_PARAMETER3;
	
	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer)); 
	CHECKERR (viPrintf (instrumentHandle, "%s %Lf", buffer, value));
	
	return rsngmo_statusQuery (instrumentHandle); 
}

/*======================================================================================*/
/* Function: Get ViReal64 with channel number											*/
/* Purpose:  This function gets command with double parameter.							*/
/*======================================================================================*/
ViStatus rsngmo_getViReal64Chan (ViSession instrumentHandle, ViString cmd, ViInt32 channel, 
							ViReal64 *value)
{
	ViUInt32    length;
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar 		buffer[BUFFER_SIZE];
	
	if (value == VI_NULL)
		return VI_ERROR_PARAMETER3;
	
	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer)); 
	CHECKERR (viPrintf (instrumentHandle, buffer));
	CHECKERR (viRead (instrumentHandle, (ViPBuf)buffer, BUFFER_SIZE, &length));
	if (length < BUFFER_SIZE)
		buffer[length] = 0;
	
	// special case of INFINITE
	if (length >= 8)
		if (!strncmp (buffer, "INFINITE", 8))
		{
			*value = 0;
			return rsngmo_statusQuery (instrumentHandle); 
		}
	sscanf (buffer,"%Le", value);
	return rsngmo_statusQuery (instrumentHandle); 
}

/*===========================================================================*/
/* Function: Set Node with channel number                                    */
/* Purpose:  This function sets simple command node that has no parameters.  */
/*===========================================================================*/
ViStatus rsngmo_setNodeChan (ViSession instrumentHandle, ViString cmd, ViInt32 channel)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar 		buffer[BUFFER_SIZE];
	
	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer)); 
	CHECKERR (viPrintf (instrumentHandle, buffer));
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*===========================================================================*/
/* Function: Get Node 		                                                 */
/* Purpose:  This function gets a string response to simple query that has   */
/*           no parameters.													 */
/*===========================================================================*/
ViStatus rsngmo_getNodeChan (ViSession instrumentHandle, ViString cmd, ViInt32 channel,
							 ViChar response[])
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViChar 		buffer[BUFFER_SIZE];
	
	if (response == VI_NULL)
		return VI_ERROR_PARAMETER3;
	
	CHECKERR (rsngmo_setChannelPart (cmd, channel, buffer)); 
	CHECKERR (viPrintf (instrumentHandle, cmd));
	CHECKERR (viScanf (instrumentHandle, "%[^\r\n]", response));  

	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Check Index List  	                                             			*/
/* Purpose:  This function checks if the passed index is valid for given list			*/
/*======================================================================================*/
ViStatus rsngmo_checkIndexInList (ViSession instrumentHandle, ViInt32 index, ViString list[], 
								   ViStatus statusIfError)
									  
{
	ViInt32		i=0;
	
	if (index < 0)
		return statusIfError;
	
	while (list[i++] != VI_NULL);
	if (index >= i)
		return statusIfError;
	
	return VI_SUCCESS;	
}

/*======================================================================================*/
/* Function: Get Index From String														*/
/* Purpose:  This function returns index for string.									*/
/*======================================================================================*/
ViStatus rsngmo_getIndexFromString (ViSession vi, ViString string, 
									 ViString *list, ViInt32 *index)
{
	ViStatus	rsngmo_status = VI_SUCCESS;
	ViInt32		i;
	size_t   	len;

	for (i = 0; list[i]; i++)
		if ((len = strlen (string)) == strlen (list[i])) {
			if (strncmp(string, list[i], strlen (string)) == 0) break;
		}	
		
	*index = i;

	return (list[i]) ? rsngmo_status : VI_ERROR_INSTR_INTERPRETING_RESPONSE;
}

/*======================================================================================*/
/* Function: Set Channel Part															*/
/* Purpose:  This function sets channel part of command									*/  
/*======================================================================================*/
ViStatus rsngmo_setChannelPart (ViString cmd, ViInt32 channel, ViChar retCmd[])
{
	ViStatus	rsngmo_status = VI_SUCCESS;
   
	if ((channel < 0) || (channel > 1))
		return VI_ERROR_PARAMETER2;

	sprintf (retCmd, cmd, channelArr[channel]);
	
	return rsngmo_status;
}

/*======================================================================================*/
/* Function: Set Active Display Channel													*/
/* Purpose:  This function sets the active channel displayed on display					*/
/*======================================================================================*/
ViStatus _setActiveDisplayChannel (ViSession instrumentHandle, ViInt32 channel)
{
	return rsngmo_setEnum (instrumentHandle, ":DISP:CHAN", channel, channelArr);
}

/*======================================================================================*/
/* Function: Set Display State															*/
/* Purpose:  This function sets the state of display									*/
/*======================================================================================*/

ViStatus _setDisplayState (ViSession instrumentHandle, ViBoolean state)
{
	return rsngmo_setViBoolean (instrumentHandle, ":DISP:ENAB", state);
}

/*======================================================================================*/
/* Function: Set Beeper State															*/
/* Purpose:  This function sets the beeper state										*/
/*======================================================================================*/
ViStatus _setBeeperState (ViSession instrumentHandle, ViInt32 channel, ViBoolean state)
{
    return rsngmo_setViBooleanChan (instrumentHandle, ":CONF:%s:CURR:LIM:BEEP", channel, state); 
}

/*======================================================================================*/
/* Function: Set/Get Data Format														*/
/* Purpose:  This function sets/gets the format of data									*/
/*======================================================================================*/
ViStatus _setDataFormat (ViSession instrumentHandle, ViInt32 format)
{
	return rsngmo_setEnum (instrumentHandle, ":FORM", format, dataFormatArr); 
}

ViStatus _getDataFormat (ViSession instrumentHandle, ViInt32 *format)
{
	return rsngmo_getEnum (instrumentHandle, ":FORM?", format, dataFormatArr); 
}

/*======================================================================================*/
/* Function: Set/Get Byte Order															*/
/* Purpose:  This function sets/gets the byte order of data								*/
/*======================================================================================*/
ViStatus _setByteOrder (ViSession instrumentHandle, ViInt32 order)
{
   return rsngmo_setEnum (instrumentHandle, ":FORM:BORD", order, dataOrderArr); 
}
ViStatus _getByteOrder (ViSession instrumentHandle, ViInt32 *order)
{
	return rsngmo_getEnum (instrumentHandle, ":FORM:BORD?", order, dataOrderArr); 
}


/*======================================================================================*/
/* Function: Set Serial State															*/
/* Purpose:  This function sets the serial interface state								*/
/*======================================================================================*/
ViStatus _setSerialState (ViSession instrumentHandle,  ViBoolean state)
{
	return rsngmo_setViBoolean (instrumentHandle, ":SYST:COMM:SER:ENAB", state);  
}

/*======================================================================================*/
/* Function: Set/Get Baud Rate															*/
/* Purpose:  This function sets/gets the serial interface baud rate						*/
/*======================================================================================*/
ViStatus _setBaudRate (ViSession instrumentHandle, ViInt32 baud)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	rsngmo_instrRange instrPtr;
	ViBoolean old_status;

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
    old_status = instrPtr -> checkStatus;
    instrPtr -> checkStatus = VI_FALSE;
	
	CHECKERR (rsngmo_setViInt32 (instrumentHandle, ":SYST:COMM:SER:BAUD", baud, 300, 38400));
	CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_ASRL_BAUD, baud));
	
	instrPtr -> checkStatus = old_status;
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Set Data Length															*/
/* Purpose:  This function sets the serial interface data length						*/
/*======================================================================================*/
ViStatus _setDataLength (ViSession instrumentHandle, ViInt32 dataLength)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	rsngmo_instrRange instrPtr;
	ViBoolean old_status;

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
    old_status = instrPtr -> checkStatus;
    instrPtr -> checkStatus = VI_FALSE;
	
	CHECKERR (rsngmo_setViInt32 (instrumentHandle, ":SYST:COMM:SER:DAT", dataLength, 7, 8));
	CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_ASRL_DATA_BITS, dataLength));
	
	instrPtr -> checkStatus = old_status;
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Set Parity																	*/
/* Purpose:  This function sets the serial interface parity								*/
/*======================================================================================*/
ViStatus _setParity (ViSession instrumentHandle, ViInt32 parity)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	rsngmo_instrRange instrPtr;
	ViBoolean old_status;

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
    old_status = instrPtr -> checkStatus;
    instrPtr -> checkStatus = VI_FALSE;
	
	CHECKERR (rsngmo_setEnum (instrumentHandle, ":SYST:COMM:SER:PAR", parity, parityArr));
	CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_ASRL_PARITY, attrParityArr[parity]));

	instrPtr -> checkStatus = old_status;
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Set Stop Bits																*/
/* Purpose:  This function sets the stop bits											*/
/*======================================================================================*/
ViStatus _setStopBits (ViSession instrumentHandle, ViInt32 stopbits)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	rsngmo_instrRange instrPtr;
	ViBoolean old_status;

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
    old_status = instrPtr -> checkStatus;
    instrPtr -> checkStatus = VI_FALSE;
	
	CHECKERR (rsngmo_setViInt32 (instrumentHandle, ":SYST:COMM:SER:STOP", stopbits, 1, 2));
	CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_ASRL_STOP_BITS, stopbits*10));

	instrPtr -> checkStatus = old_status;
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Set Handshake																*/
/* Purpose:  This function sets the handshake											*/
/*======================================================================================*/
ViStatus _setHandshake (ViSession instrumentHandle, ViInt32 handshake)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	rsngmo_instrRange instrPtr;
	ViBoolean old_status;

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));
    old_status = instrPtr -> checkStatus;
    instrPtr -> checkStatus = VI_FALSE;
	
	CHECKERR (rsngmo_setEnum (instrumentHandle, ":SYST:COMM:SER:HAND", handshake, handshakeArr));
	CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_ASRL_FLOW_CNTRL, attrHandshakeArr[handshake]));
	
	instrPtr -> checkStatus = old_status;
	
	return rsngmo_statusQuery (instrumentHandle);
}

/*======================================================================================*/
/* Function: Set Delimiter																*/
/* Purpose:  This function sets the delimiter											*/
/*======================================================================================*/
ViStatus _setDelimiter (ViSession instrumentHandle, ViInt32 delimiter)
{
	return rsngmo_setEnum (instrumentHandle, ":SYST:COMM:SER:DEL", delimiter, delimiterArr);
}

/*======================================================================================*/
/* Function: Set Output State															*/
/* Purpose:  This function sets the state of selected output							*/
/*======================================================================================*/
ViStatus _setOutputState (ViSession instrumentHandle, ViInt32 channel, ViBoolean state)
{
	return rsngmo_setViBooleanChan (instrumentHandle, ":OUTP:%s:STAT", channel, state);
}

/*======================================================================================*/
/* Function: Set Output Bandwidth														*/
/* Purpose:  This function sets the output bandwidth									*/
/*======================================================================================*/
ViStatus _setOutputBandwidth (ViSession instrumentHandle, ViInt32 channel, ViInt32 bandwidth)
{
	return rsngmo_setEnumChan (instrumentHandle, ":OUTP:%s:BAND", channel, bandwidth, bandwidthArr);  
}

/*======================================================================================*/
/* Function: Set Output Impedance														*/
/* Purpose:  This function sets the output impedance									*/
/*======================================================================================*/
ViStatus _setOutputImpedance (ViSession instrumentHandle, 
								ViInt32 channel, ViReal64 impedance)
{
	return rsngmo_setViReal64Chan (instrumentHandle, ":OUTP:%s:IMP", channel, impedance, 0, 1);    
}

/*======================================================================================*/
/* Function: Set Voltage Level															*/
/* Purpose:  This function sets the source voltage amplitude							*/
/*======================================================================================*/
ViStatus _setVoltageLevel (ViSession instrumentHandle, 
							ViInt32 channel, ViReal64 voltageLevel)
{
	return rsngmo_setViReal64Chan (instrumentHandle, ":SOUR:%s:VOLT", 
										channel, voltageLevel, 0.0, 15.0); 	
}

/*======================================================================================*/
/* Function: Set Voltage Protection														*/
/* Purpose:  This function sets the source voltage protection							*/
/*======================================================================================*/
ViStatus _setVoltageProtection (ViSession instrumentHandle, ViInt32 channel,
                                               ViReal64 protectionLevel)
{
		return rsngmo_setViReal64Chan (instrumentHandle, ":SOUR:%s:VOLT:PROT", 
										channel, protectionLevel, 1.5, 22.0); 	
    
}

/*======================================================================================*/
/* Function: Set Voltage Max  Setting													*/
/* Purpose:  This function sets the max voltage settable value							*/
/*======================================================================================*/
ViStatus _setVoltageMaxSetting (ViSession instrumentHandle, ViInt32 channel, ViReal64 max)
{
	return rsngmo_setViReal64Chan (instrumentHandle, ":SOUR:%s:VOLT:MAXS", 
										channel, max, 0.0, 15.0); 	
}

/*======================================================================================*/
/* Function: Set Current Limit															*/
/* Purpose:  This function sets the current limit										*/
/*======================================================================================*/
ViStatus _setCurrentLimit (ViSession instrumentHandle, ViInt32 channel, ViReal64 limit)
{
	return rsngmo_setViReal64Chan (instrumentHandle, ":SOUR:%s:CURR", channel,
									limit, 0.0, 5.0); 	
}

/*======================================================================================*/
/* Function: Set/Get Current Limit	Max Setting											*/
/* Purpose:  This function sets/gets the maximum settable current limit					*/
/*======================================================================================*/
ViStatus _setCurrentLimitMaxSetting (ViSession instrumentHandle, ViInt32 channel, 
										ViReal64 max)
{
	return rsngmo_setViReal64Chan (instrumentHandle, ":SOUR:%s:CURR:MAXS", channel,
									max, 0.0, 5.0); 	
}

/*======================================================================================*/
/* Function: Set Current Limit Type														*/
/* Purpose:  This function sets the current limit type									*/
/*======================================================================================*/
ViStatus _setCurrentLimitType (ViSession instrumentHandle, ViInt32 channel, ViInt32 type)
{
	return rsngmo_setEnumChan (instrumentHandle, ":SOUR:%s:CURR:TYPE", channel, type,
								limitTypeArr);    
}

/*======================================================================================*/
/* Function: Set Measurement Function													*/
/* Purpose:  This function sets the measurement function								*/
/*======================================================================================*/
ViStatus _setMeasurementFunction (ViSession instrumentHandle, ViInt32 channel, 
                                  ViInt32 function)
{
	return rsngmo_setEnumChan (instrumentHandle, ":SENS:%s:FUNC", channel, function,
								functionArr);
}

/*======================================================================================*/
/* Function: Set Current Measurement Range												*/
/* Purpose:  This function sets expected current measurement range						*/
/*======================================================================================*/
ViStatus _setCurrentMeasurementRange (ViSession instrumentHandle, ViInt32 channel,
                                     ViInt32 range)
{
	return rsngmo_setEnumChan (instrumentHandle, ":SENS:%s:CURR:RANGE", channel, range,
							currRangeArr);
}

/*======================================================================================*/
/* Function: Set Measurement Interval													*/
/* Purpose:  This function sets/gets measurement interval for voltage and current		*/
/*======================================================================================*/
ViStatus _setMeasurementInterval (ViSession instrumentHandle, ViInt32 channel,
                                  ViReal64 interval)
{
	return rsngmo_setViReal64Chan (instrumentHandle, ":SENS:%s:MEAS:INT", channel,
									interval, 2.00E-3, 2.00E-1);   
}

/*======================================================================================*/
/* Function: Set Average Count															*/
/* Purpose:  This function sets the measure average count								*/
/*======================================================================================*/
ViStatus _setAverageCount (ViSession instrumentHandle, ViInt32 channel, 
							ViInt32 averageCount)
{
	return rsngmo_setViInt32Chan (instrumentHandle, ":SENS:%s:AVER:COUN", channel,
								averageCount, 1, 10);    
}

/*======================================================================================*/
/* Function: Set Pulse Channel															*/
/* Purpose:  This function sets the pulse measurement channel.							*/
/*======================================================================================*/
ViStatus _setPulseChannel (ViSession instrumentHandle, ViInt32 channel, 
							ViInt32 pulseChannel)
{
	return rsngmo_setEnumChan (instrumentHandle, ":SENS:%s:PULS:CHAN", channel, pulseChannel,
								pulseChanArr);      
}

/*======================================================================================*/
/* Function: Set Pulse State															*/
/* Purpose:  This function sets the pulse measurement state.							*/
/*======================================================================================*/
ViStatus _setPulseState (ViSession instrumentHandle, ViInt32 channel, ViBoolean state)
{
	return rsngmo_setViBooleanChan (instrumentHandle, ":SENS:%s:PULS:STAR", channel, state);    
}

/*======================================================================================*/
/* Function: Set Pulse Type																*/
/* Purpose:  This function sets the pulse measurement type.								*/
/*======================================================================================*/
ViStatus _setPulseType (ViSession instrumentHandle, ViInt32 channel, ViInt32 type)
{
	if (type > RSNGMO_FUNC_RMS)
		return VI_ERROR_PARAMETER3;
		
	return rsngmo_setEnumChan (instrumentHandle, ":SENS:%s:PULS:TYPE", channel, type, functionArr);    
}

/*======================================================================================*/
/* Function: Set Pulse Trigger Level													*/
/* Purpose:  This function sets the pulse trigger level									*/
/*======================================================================================*/
ViStatus _setPulseTriggerLevel (ViSession instrumentHandle, ViInt32 channel,
                                ViInt32 triggerRange, ViReal64 triggerLevel)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];

	if (rsngmo_invalidViInt32Range (triggerRange, 0, 2))
		return VI_ERROR_PARAMETER3;
	
	CHECKERR (rsngmo_setChannelPart (":SENS:%s:PULS:TRIG:LEV:", channel, buffer)); 
	strcat (buffer, triggerRangeArr[triggerRange]);
	CHECKPARM (rsngmo_setViReal64 (instrumentHandle, buffer, triggerLevel, minTriggerRangeArr[triggerRange],
									maxTriggerRangeArr[triggerRange]), VI_ERROR_PARAMETER4);
	
	return rsngmo_status;
}

/*======================================================================================*/
/* Function: Set Pulse Trigger Source													*/
/* Purpose:  This function sets the pulse trigger source									*/
/*======================================================================================*/
ViStatus _setPulseTriggerSource (ViSession instrumentHandle, ViInt32 channel, ViInt32 source)
{
	return rsngmo_setEnumChan (instrumentHandle, ":SENS:%s:PULS:TRIG:SOUR", channel,
								source, triggerSourceArr);    
}

/*======================================================================================*/
/* Function: Set Pulse Trigger Slope													*/
/* Purpose:  This function sets the pulse trigger slope									*/
/*======================================================================================*/
ViStatus _setPulseTriggerSlope (ViSession instrumentHandle, ViInt32 channel, ViInt32 slope)
{
	return rsngmo_setEnumChan (instrumentHandle, ":SENS:%s:PULS:TRIG:SLOP", channel, 
								slope, triggerSlopeArr);    
}

/*======================================================================================*/
/* Function: Set Pulse Trigger Offset													*/
/* Purpose:  This function sets the pulse trigger offset								*/
/*======================================================================================*/
ViStatus _setPulseTriggerOffset (ViSession instrumentHandle, ViInt32 channel, ViInt32 offset)
{
	return rsngmo_setViInt32Chan (instrumentHandle, ":SENS:%s:PULS:TRIG:OFFS", channel,
									offset, -5000, 50000);    
}

/*======================================================================================*/
/* Function: Set Pulse Trigger Count													*/
/* Purpose:  This function sets the pulse trigger count									*/
/*======================================================================================*/
ViStatus _setPulseTriggerCount (ViSession instrumentHandle, ViInt32 channel, ViInt32 triggerCount)
{
	return rsngmo_setViInt32Chan (instrumentHandle, ":SENS:%s:PULS:TRIG:COUN",
									channel, triggerCount, 1, 100);       
}

/*======================================================================================*/
/* Function: Set Pulse Trigger Timeout													*/
/* Purpose:  This function sets the pulse trigger timrout								*/
/*======================================================================================*/
ViStatus _setPulseTriggerTimeout (ViSession instrumentHandle,
									ViInt32 channel,ViReal64 timeout)
{
	return rsngmo_setViReal64Chan (instrumentHandle, ":SENS:%s:PULS:TRIG:TIM", channel,
									timeout, 0.0, 60.0);     
}

/*======================================================================================*/
/* Function: Set/Get Pulse Number Of Samples											*/
/* Purpose:  This function sets/gets the pulse number of samples						*/
/*======================================================================================*/
ViStatus _setPulseNumberOfSamples (ViSession instrumentHandle,
                                    ViInt32 channel, ViInt32 numberOfSamples)
{
	return rsngmo_setViInt32Chan (instrumentHandle, ":SENS:%s:PULS:SAMP:LENG", channel,
									numberOfSamples, 50, 5000);      
}

/*======================================================================================*/
/* Function: Set Pulse Sample Interval													*/
/* Purpose:  This function sets the pulse sample interval								*/
/*======================================================================================*/
ViStatus _setPulseSampleInterval (ViSession instrumentHandle,
                                  ViInt32 channel, ViReal64 sampleInterval)
{
	return rsngmo_setViReal64Chan (instrumentHandle, ":SENS:%s:PULS:SAMP:INT", channel,
									sampleInterval, 9.0E-6, 1.0);    
}

/*======================================================================================*/
/* Function: Get Data Array																*/
/* Purpose:  This function returns binary data from the instrument.						*/
/*======================================================================================*/
ViStatus rsngmo_getDataArray (ViSession instrumentHandle, ViChar cmd[], ViInt32 channel,
							ViInt32 function, ViInt32 numberPoints, ViReal64 *array, ViInt32 *retCount)
{
	ViStatus			rsngmo_status = VI_SUCCESS;
	ViUInt16			interface;     
	ViInt32				numOfBytes;
	ViUInt16			endIn = '\n';
	ViBoolean			termEn = VI_TRUE;
	ViInt32				i;
	ViInt32				retCnt;
	ViInt32				measStatus = 0;
	ViInt32				bufferLength = 1;
	ViReal32			*p;
	ViReal64			tim_1;
	rsngmo_instrRange	instrPtr;
	ViBoolean			old_status;
	ViChar				buffer[20100];		// max values 5000 * 4 (bytes) + rest
	
	if (function == VI_NULL)
		strcpy (buffer, cmd);
	else
	{
		CHECKERR (rsngmo_checkIndexInList (instrumentHandle, function, functionArr, VI_ERROR_PARAMETER3));
		strcpy (buffer, cmd);
		strcat (buffer, ":"); 
		strcat (buffer, functionArr[function]);
	}		

    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));

	if (numberPoints > 1)
	{
		strcat (buffer, ":ARR?\n"); 
		// get size of instrument's buffer
		if (instrPtr->checkStatus)
			CHECKERR (rsngmo_getPulseNumberOfSamples (instrumentHandle, channel, &bufferLength));
		else
			bufferLength = numberPoints;
	}
	else
		strcat (buffer, "?\n");
	
	if (instrPtr->checkStatus)
		CHECKERR(rsngmo_getPulseTriggerTimeout(instrumentHandle, channel, &tim_1));
	else
		tim_1 = 0.0;

	// turn off error checking
    old_status = instrPtr -> checkStatus;
    instrPtr -> checkStatus = VI_FALSE;
	
	CHECKERR(rsngmo_setNodeChan (instrumentHandle, buffer, channel));
    
	// set the Error Checking to its previous state
    instrPtr -> checkStatus = old_status;

	CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_INTF_TYPE, &interface));
	
	/**********************************/
	/* NOTE:                          */
	/* If interface is serial port,   */
	/* disable termination character. */
	/**********************************/
	if (interface == VI_INTF_ASRL){
		CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_ASRL_END_IN, &endIn));     
		CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
		CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_TERMCHAR_EN, &termEn));
		CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_TERMCHAR_EN, VI_FALSE));  
	}
	buffer[0] = '\0';
	numOfBytes = (numberPoints > 1) ? (2 + bufferLength*4 + 1) : (2 + numberPoints*4 + 1);
	
	// wait until measurement is done
	if (strncmp(cmd,"FETC",4))
	{
		ViUInt32 t_out = 0;
		ViStatus read_status;
		if (tim_1)
			{
			CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_TMO_VALUE, &t_out));
			CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_TMO_VALUE, (unsigned long) tim_1*1000 + 500));
			}
	
   		read_status = viRead (instrumentHandle, (ViPBuf)buffer, numOfBytes,(ViPUInt32) &retCnt);
		
		if (tim_1)
			CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_TMO_VALUE, t_out));
		
		if (interface == VI_INTF_ASRL)
		{
			CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_ASRL_END_IN, endIn));
			CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_TERMCHAR_EN, termEn));  
		}
		
		if (instrPtr->checkStatus)
		{
			rsngmo_status = rsngmo_getStatusRegister (instrumentHandle, RSNGMO_REG_MEAS_EVENT,
													  &measStatus);

			if (measStatus & 0x0090)
				return RSNGMO_ERROR_TRIGGER_TIMEOUT; 
		}
		
		if (read_status != VI_SUCCESS)
			return RSNGMO_ERROR_MEASUREMENT_TIMEOUT;

	}
	else
	{
		//startdata + data + LF (data in SREAL format 32bits)
	   	CHECKERR (viRead (instrumentHandle, (ViPBuf)buffer, numOfBytes, (ViPUInt32)&retCnt));
		/**********************************/
		/* NOTE:                          */
		/* If interface is serial port,   */
		/* enable termination character.  */
		/**********************************/
		if (interface == VI_INTF_ASRL){
			CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_ASRL_END_IN, endIn));
			CHECKERR (viSetAttribute (instrumentHandle, VI_ATTR_TERMCHAR_EN, termEn));  
		}
   	}
   	

   	if (retCnt != numOfBytes)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	CHECKERR (viFlush (instrumentHandle, VI_READ_BUF_DISCARD));

	retCnt = (retCnt-3)/4;
	*retCount = (retCnt < numberPoints) ? retCnt : numberPoints;
	p = (ViReal32*)(buffer + 2);
	
	for (i=0; i<*retCount; i++)
		array[i] = p[i];	
		
	return rsngmo_statusQuery (instrumentHandle);	
	
}
/*======================================================================================*/
/* Function: Get Data Single															*/
/* Purpose:  This function returns a reading											*/
/*======================================================================================*/
ViStatus rsngmo_getDataSingle (ViSession instrumentHandle, ViChar cmd[], ViInt32 channel,
							ViInt32 function, ViReal64 *value)
{
	ViReal64 arr[1];
	ViInt32 retCount;
	ViStatus rsngmo_status;
	
	CHECKERR (rsngmo_getDataArray (instrumentHandle, cmd, channel, function, 1, arr, &retCount));     
	*value = arr[0];
	return rsngmo_status;
}

/*======================================================================================*/
/* Function: Set Status Register														*/
/* Purpose:  This function write a register of the NGMO2								*/
/*======================================================================================*/
ViStatus _setStatusRegister (ViSession instrumentHandle, ViInt32 reg, ViInt32 value)
{
	ViStatus rsngmo_status = VI_SUCCESS;
	
	if (rsngmo_invalidViInt32Range (reg, 0, 4))
		return VI_ERROR_PARAMETER2;
	
	CHECKPARM (rsngmo_setViInt32 (instrumentHandle, registerArr[reg], value, 0, 32767),
				VI_ERROR_PARAMETER3);
	return rsngmo_status;				
}

/*======================================================================================*/
/* Function: Delay              														*/
/* Purpose:  This function delays without use of utility.h  							*/
/*======================================================================================*/
void rsngmo_delay (ViReal64 delay)
{
	time_t time_first, time_actual;
	
	time (&time_first);
	time_actual = time_first;
	while (time_actual < (time_first+delay))
		time (&time_actual);
}

/****************************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE ================================================*/
/****************************************************************************************/
	   
